/*****************************************************************************
 * amx_ejl_fireworks.sma     version 1.1                  Date: 3/26/2003
 *  Author: Eric Lidman      ejlmozart@hotmail.com
 *  Alias: Ludwig van        Upgrade: http://lidmanmusic.com/cs/plugins.html    
 *
 *  Shoot fireworks in a variety of ways and effects with this plugin. 
 *   There are options for automatically setting them at random points in the
 *   map or you may set them at specific points. Admins can control the 
 *   effects. The effects look best on bigger more open maps. Nippers map for
 *   CS: helms_deep is an ideal example, but they look good on other maps as
 *   well. The plugin contains automatic spam contols so you dont crash your
 *   server with all the effects, as tempting as that might be. The plugin 
 *   was written for the CS mod primarily.     
 *
 *  ADMIN COMMANDS:
 *
 *   amx_fireworks_set <type 0-5 or cancell> <height> <quantity> <delay>
 *                     The main and certainly most fun command ths plugin 
 *                     offers. Sets or cancels local fireworks displays.
 *   amx_fireworks     <time in seconds or 0 to stop>
 *                     Finale style! Sets four types fireworks off at random
 *                     points on map.
 *   amx_fireworks1    <time in seconds or 0 to stop>
 *                     Sets debris model fireworks off at random points on map
 *   amx_fireworks2    <time in seconds or 0 to stop>
 *                     Sets quake1 expolsion fireworks at random points on map
 *   amx_fireworks3    <time in seconds or 0 to stop>
 *                     Sets particle burst fireworks at random points on map
 *   amx_fireworks4    <time in seconds or 0 to stop>
 *                     Roman candle -- not done yet, sorry.
 *   amx_fireworks5    <time in seconds or 0 to stop>
 *                     Sets glow sprite fireworks off at random points on map
 *   amx_fireworks_lc  : toggles whether type 1 and 5 fireworks explode low on
 *                     maps with a low ceiling. The problem with these types
 *                     is they need to explode in the map to be seen.
 *   amx_fireworks_sound   : toggles fireworks noise on and off. If you like 
 *                         fireworks and want everyone else to like them, this
 *                         command can help. After a short time, all the noise
 *                         from fireworks can get annoying, yet the visuals are
 *                         still appealing, so turn off the sound with this. 
 *
 *  CVARS (which can be set in the admin.cfg):
 *
 * 	amx_fireworks_noise 1     -- 0 or 1, this sets whether your fireworks
 *                              make noise by default.
 *	amx_fireworks_roundend 0  -- 0 = no fireworks display at round end in CS
 *                              -- 1 = fireworks at round end, but no sound
 *                              -- 2 = fireworks at round end, with sound
 *	  
 *  ADDITIONAL SETUP:
 *
 *   Some maps like de_dust and cs_office (CS maps) have low ceilings, not much
 *    room for fireworks to display well. This is particularly a problem with
 *    the fireworks that use glowsprites and models. They dont display once they
 *    pass the ceiling. This plugin automatically sets the low cieling command 
 *    on certain maps, but more can be added as needed to this file:
 *    "fireworks.ini" which can be found in the ..addons/amx folder. The plugin
 *    automatically on first run makes the file. Use that. Just add the name
 *    of the map. One map name per line.
 *
 *  NOTE:
 *
 *   This plugin has room to grow. If you have ideas, let me know. Perhaps we
 *    can set off fireworks automatically on statsme type events like multikill
 *    or headshot. The possiblilities for attaching fireworks to game events
 *    are limitless. Check back often as new effects and options will be added.
 *    Also, this is a stand alone plugin for AMXMOD, you do not need the
 *    metamod fireworks plugin to run this one. If you want, you may however 
 *    run them at the same time, but they are not connected in any way.
 * 
 *****************************************************************************/

#include <amxmod>
#include <amxmisc>

new l_tube
new garbgibs
new flare3
new flare6
new blueflare2
new redflare2
new smoke 
new mflash
new mapadjust
new bool:lowceiling = false
new bigshow[33]
new antispam
new antispam2
new antispam3
new DoOnce
new bool:dod_running

public amx_fw_set(id,level,cid){
	// args (x,y,z,type,height,count,fuse,dirvx,dirvy)
	if (!cmd_access(id,level,cid,2))
		return PLUGIN_HANDLED

	new arg1[8],arg2[8],arg3[8],arg4[8],origin[3],args[8],aimv[3]
	new iarg1,iarg2,iarg3,iarg4,dirvx,dirvy
	read_argv(1,arg1,7)
	read_argv(2,arg2,7)
	read_argv(3,arg3,7)
	read_argv(4,arg4,7)

	if(equali(arg1,"c",1)){
		remove_task(6520)
		remove_task(6521)
		remove_task(6522)
		remove_task(6341)
		remove_task(6342)
		remove_task(6343)
		remove_task(6344)
		console_print(id,"[AMX] Fireworks cancelled")
		return PLUGIN_HANDLED
	}

	if(!(get_user_flags(id) & ADMIN_LEVEL_B)){
		if( (id != 0) && (antispam > 4) ){
			console_print(id,"[AMX] You dont have enough access to use this command unlimitedly. Please wait.")
			return PLUGIN_HANDLED
		}
		antispam += 1
	}
	antispam3 +=1
	if(antispam3 > 6){
		console_print(id,"[AMX] Now, now ... lets not spam. Wait till next round.")
		return PLUGIN_HANDLED
	}

	if(id < 33 && id > 0){		
		get_user_origin(id,origin)
		get_user_origin(id,aimv,2)
		if(origin[0] > aimv[0])
			dirvx = -50
		else
			dirvx = 50
		if(origin[1] > aimv[1])
			dirvy = -50
		else
			dirvy = 50
	}
	iarg1 = strtonum(arg1)
	iarg2 = strtonum(arg2)
	iarg3 = strtonum(arg3)
	iarg4 = strtonum(arg4)
	if(iarg1 != 0 && iarg1 != 1 && iarg1 != 2 && iarg1 != 3 && iarg1 != 5) 
		iarg1 = 0
	if(iarg2 < 1 || iarg2 > 10000)
		iarg2 = 1000
	if(iarg3 < 1 || iarg3 > 100) 
		iarg3 = 1
	if(iarg4 < 1 || iarg4 > 60)
		iarg4 = 5 
	args[0] = origin[0] + dirvx
	args[1] = origin[1] + dirvy
	args[2] = origin[2]
	args[3] = iarg1   
	args[4] = iarg2
	args[5] = iarg3
	args[6] = dirvx
	args[7] = dirvy

	set_task(iarg4 + 0.0,"fw_set_fuse",6520,args,8,"a",0)
	emit_sound(id,CHAN_ITEM, "weapons/grenade_hit2.wav", 1.0, ATTN_NORM, 0, PITCH_NORM)
	refresh_tube(args)

	new tt = (iarg3 * 2) + iarg4 + 2
	new tt1 = tt / 25
	if(tt > 25)
		set_task(25.0,"refresh_tube",6521,args,8,"a",tt1 - 1)

	return PLUGIN_HANDLED
}

public refresh_tube(args[]){

	// TE_MODEL from HL-SDK common/const.h 
	message_begin(MSG_ALL,SVC_TEMPENTITY) 
	write_byte(106) 
	write_coord(args[0] + args[6]) 
	write_coord(args[1] + args[7])   
	write_coord(args[2] - 30) 
	write_coord(0) 
	write_coord(0) 
	write_coord(0) 
	write_angle (0) 
	write_short (l_tube) 
	write_byte (0) 
	write_byte (255) 
	message_end()

	return PLUGIN_CONTINUE
}

public fw_set_fuse(args[]){
	set_task(2.0,"fw_set_mid",6522,args,8,"a",args[5]-1)
	return PLUGIN_CONTINUE
}

public fw_set_mid(args[]){
	new rn
	if(get_cvar_num("amx_fireworks_noise")==1){
		rn = random_num(0,1)
		if(rn == 1){
			client_cmd(0, "spk ambience/biggun3.wav")
		}else{
			client_cmd(0, "spk ambience/biggun2.wav")
		}
	} 

	new rx = random_num(-250,250)
	new ry = random_num(-250,250)

	//TE_TRACER	
	message_begin( MSG_BROADCAST,SVC_TEMPENTITY) 
	write_byte( 6 ) // tracer effect from point to point
	write_coord(args[0] + args[6]) 
	write_coord(args[1] + args[7]) 
	write_coord(args[2]) 
	write_coord(args[0] + rx) 
	write_coord(args[1] + ry) 
	write_coord(args[2]+args[4])
	message_end()

	//TE_SPRITE
	message_begin( MSG_BROADCAST,SVC_TEMPENTITY) 
	write_byte( 17 ) // additive sprite, plays 1 cycle
	write_coord(args[0] + args[6])  
	write_coord(args[1] + args[7])  
	write_coord(args[2] - 10 ) 
	write_short( mflash ) 
	write_byte( 10 ) // byte (scale in 0.1's)  
	write_byte( 255 ) //  byte (brightness) 
	message_end()
  
	//TE_SMOKE 
	message_begin( MSG_BROADCAST,SVC_TEMPENTITY) 
	write_byte( 5 )
	write_coord(args[0] + args[6]) 
	write_coord(args[1] + args[7]) 
	write_coord(args[2]+35) 
	write_short( smoke )
	write_byte( 40 )
	write_byte( 10 )  
	message_end()

	new args2[3]
	args2[0] = args[0] + rx
	args2[1] = args[1] + ry
	args2[2] = args[2] + args[4]

	if(args[3] == 0){
		rn = random_num(0,3)
		switch(rn){
			case 0: set_task(0.4,"fw_garbage1",0,args2,3,"a",0)
			case 1: set_task(0.4,"fw_tara1",0,args2,3,"a",0)
			case 2: set_task(0.5,"fw_spr1",0,args2,3,"a",0)
			case 3: set_task(0.5,"fw_ooo1",0,args2,3,"a",0)
		}
	}
	else if(args[3] == 1)
		set_task(0.4,"fw_garbage1",0,args2,3,"a",0)
	else if(args[3] == 2)
		set_task(0.4,"fw_tara1",0,args2,3,"a",0)
	else if(args[3] == 5)
		set_task(0.5,"fw_spr1",0,args2,3,"a",0)
	else if(args[3] == 3)
		set_task(0.5,"fw_ooo1",0,args2,3,"a",0)

	return PLUGIN_HANDLED
}


public amx_fw_g(id,level,cid){
	if (!cmd_access(id,level,cid,2))
		return PLUGIN_HANDLED
	remove_task(6341)
	new arg1[8],iarg1
	read_argv(1,arg1,7)
	iarg1 = strtonum(arg1)
	if(iarg1 > 300)
		iarg1 = 300
	if(iarg1 < 1){
		client_print(0,print_chat,"[AMX]  Awww, no more fireworks.")
	}else{
		client_print(0,print_chat,"[AMX]  Oooooo! Ahhhhh!  Fireworks!")
		set_task(2.0,"fw_garbage",6341,"",0,"a",iarg1/2)
	}
	return PLUGIN_HANDLED
}

public fw_garbage(){
	new xy[3]
	xy[0] = random_num(-2200,2200)
	xy[1] = random_num(-2200,2200)
	xy[2] = random_num(100,1400) + mapadjust
	if(lowceiling == true)
		xy[2] = mapadjust + 100		
	if(get_cvar_num("amx_fireworks_noise")==1)
		client_cmd(0, "spk ambience/biggun3.wav")
 
	//TE_TRACER	
	message_begin( MSG_BROADCAST,SVC_TEMPENTITY) 
	write_byte( 6 ) // tracer effect from point to point
	write_coord(xy[0]) 
	write_coord(xy[1]) 
	write_coord(-1000 + mapadjust) 
	write_coord(xy[0]) 
	write_coord(xy[1]) 
	write_coord(xy[2])
	message_end()  

	set_task(0.4,"fw_garbage1",0,xy,3,"a",0)

	return PLUGIN_HANDLED
}

public fw_garbage1(xy[]){
	new velo = random_num(300,700)
	if(get_cvar_num("amx_fireworks_noise")==1)
		client_cmd(0, "spk weapons/explode5.wav")

	//define TE_EXPLODEMODEL
	message_begin(MSG_BROADCAST,SVC_TEMPENTITY)
	write_byte(107) // spherical shower of models, picks from set
	write_coord(xy[0]) // pos
	write_coord(xy[1])  
	write_coord(xy[2]) 
	write_coord(velo) //(velocity)
	write_short (garbgibs) //(model index)
	write_short (25) // (count)
	write_byte (15) // (life in 0.1's)		
	message_end()

	return PLUGIN_HANDLED
}

public amx_fw_tara(id,level,cid){
	if (!cmd_access(id,level,cid,2))
		return PLUGIN_HANDLED
	remove_task(6342)
	new arg1[8],iarg1
	read_argv(1,arg1,7)
	iarg1 = strtonum(arg1)
	if(iarg1 > 300)
		iarg1 = 300
	if(iarg1 < 1){
		client_print(0,print_chat,"[AMX]  Awww, no more fireworks.")
	}else{
		client_print(0,print_chat,"[AMX]  Oooooo! Ahhhhh!  Fireworks!")
		set_task(2.0,"fw_tara",6342,"",0,"a",iarg1/2)
	}

	return PLUGIN_HANDLED
}

public fw_tara(){
	new xy[3]
	xy[0] = random_num(-2200,2200)
	xy[1] = random_num(-2200,2200)
	xy[2] = random_num(100,1900) + mapadjust
	if(get_cvar_num("amx_fireworks_noise")==1)
		client_cmd(0, "spk ambience/biggun2.wav")

	//TE_TRACER	
	message_begin( MSG_BROADCAST,SVC_TEMPENTITY) 
	write_byte( 6 ) // tracer effect from point to point
	write_coord(xy[0]) 
	write_coord(xy[1]) 
	write_coord(-1000 + mapadjust) 
	write_coord(xy[0]) 
	write_coord(xy[1]) 
	write_coord(xy[2])
	message_end() 

	set_task(0.4,"fw_tara1",0,xy,3,"a",0)

	return PLUGIN_HANDLED
}

public fw_tara1(xy[]){
	if(get_cvar_num("amx_fireworks_noise")==1)
		client_cmd(0, "spk weapons/explode4.wav")
	//TE_TAREXPLOSION
	message_begin(MSG_BROADCAST,SVC_TEMPENTITY)
	write_byte( 4) // Quake1 "tarbaby" explosion with sound
	write_coord(xy[0]) // pos
	write_coord(xy[1])  
	write_coord(xy[2]) 
	message_end()

	return PLUGIN_HANDLED
}

public amx_fw_spr(id,level,cid){
	if (!cmd_access(id,level,cid,2))
		return PLUGIN_HANDLED
	remove_task(6344)
	new arg1[8],iarg1
	read_argv(1,arg1,7)
	iarg1 = strtonum(arg1)
	if(iarg1 > 300)
		iarg1 = 300
	if(iarg1 < 1){
		client_print(0,print_chat,"[AMX]  Awww, no more fireworks.")
	}else{
		client_print(0,print_chat,"[AMX]  Oooooo! Ahhhhh!  Fireworks!")
		set_task(2.0,"fw_spr",6344,"",0,"a",iarg1/2)
	}

	return PLUGIN_HANDLED
}

public fw_spr(){
	new xy[3]
	xy[0] = random_num(-2200,2200)
	xy[1] = random_num(-2200,2200)
	xy[2] = random_num(100,1400) + mapadjust
	if(lowceiling == true)
		xy[2] = mapadjust + 100		
	if(get_cvar_num("amx_fireworks_noise")==1)
		client_cmd(0, "spk ambience/biggun3.wav")

	//TE_TRACER	
	message_begin( MSG_BROADCAST,SVC_TEMPENTITY) 
	write_byte( 6 ) // tracer effect from point to point
	write_coord(xy[0]) 
	write_coord(xy[1]) 
	write_coord(-1000 + mapadjust) 
	write_coord(xy[0]) 
	write_coord(xy[1]) 
	write_coord(xy[2])
	message_end() 
	
	set_task(0.5,"fw_spr1",0,xy,3,"a",0)

	return PLUGIN_HANDLED
}

public fw_spr1(xy[]){
	new velo = random_num(40,100)
	new spr
	new choosespr = random_num(0,3) 
	if(choosespr == 0)
		spr = flare3
	else if(choosespr == 1)
		spr = blueflare2
	else if(choosespr == 2)
		spr = flare6
	else if(choosespr == 3)
		spr = redflare2
	if(get_cvar_num("amx_fireworks_noise")==1)
		client_cmd(0, "spk weapons/explode4.wav")

	//TE_SPRITETRAIL
	message_begin(MSG_BROADCAST,SVC_TEMPENTITY)
	write_byte (15)	// line of moving glow sprites with gravity, fadeout, and collisions
	write_coord(xy[0]) // pos
	write_coord(xy[1])  
	write_coord(xy[2]) 
	write_coord(xy[0]) // pos
	write_coord(xy[1])  
	write_coord(xy[2]+10) 
	write_short(spr) // (sprite index)
	write_byte(50) // (count)
	write_byte(1) // (life in 0.1's) 
	write_byte(10) // byte (scale in 0.1's) 
	write_byte(velo) // (velocity along vector in 10's)
	write_byte(40) // (randomness of velocity in 10's)

	message_end()
}


public amx_fw_rc(id,level,cid){
	if (!cmd_access(id,level,cid,1))
		return PLUGIN_HANDLED

	new origin[3]
	get_user_origin(id,origin)

	set_task(3.0,"fw_rc1",0,origin,3,"a",0)

	return PLUGIN_HANDLED
}

public fw_rc1(origin[]){

	//TE_SPRAY	
	message_begin(MSG_BROADCAST,SVC_TEMPENTITY)
	write_byte (120) // Throws a shower of sprites or models
	write_coord(origin[0]) // pos
	write_coord(origin[1]) 
	write_coord(origin[2])
	write_coord(10) //origin[0]) // directtion
	write_coord(10) //origin[1]) 
	write_coord(150) //origin[2]+400)
	write_short (flare6) // spr
	write_byte (10) // count
	write_byte (15) // byte (speed)
	write_byte (0) //(noise)
	write_byte (5) // (rendermode)
	message_end()

	return PLUGIN_HANDLED
}

public amx_fw_ooo(id,level,cid){
	if (!cmd_access(id,level,cid,2))
		return PLUGIN_HANDLED

	remove_task(6343)
	new arg1[8],iarg1
	read_argv(1,arg1,7)
	iarg1 = strtonum(arg1)
	if(iarg1 > 300)
		iarg1 = 300
	if(iarg1 < 1){
		client_print(0,print_chat,"[AMX]  Awww, no more fireworks.")
	}else{
		client_print(0,print_chat,"[AMX]  Oooooo! Ahhhhh!  Fireworks!")
		set_task(2.0,"fw_ooo",6343,"",0,"a",iarg1/2)
	}

	return PLUGIN_HANDLED
}

public fw_ooo(){
	new xy[3]
	xy[0] = random_num(-2200,2200)
	xy[1] = random_num(-2200,2200)
	xy[2] = random_num(100,1900) + mapadjust
	if(get_cvar_num("amx_fireworks_noise")==1)
		client_cmd(0, "spk ambience/biggun3.wav")

	//TE_TRACER	
	message_begin( MSG_BROADCAST,SVC_TEMPENTITY) 
	write_byte( 6 ) // tracer effect from point to point
	write_coord(xy[0]) 
	write_coord(xy[1]) 
	write_coord(-1000 + mapadjust) 
	write_coord(xy[0]) 
	write_coord(xy[1]) 
	write_coord(xy[2])
	message_end() 

	set_task(0.5,"fw_ooo1",0,xy,3,"a",0)

	return PLUGIN_HANDLED
}

public fw_ooo1(xy[]){
	new color = random_num(0,255)
	new width = random_num(400,1000)
	if(get_cvar_num("amx_fireworks_noise")==1)
		client_cmd(0, "spk weapons/explode3.wav")

	//TE_PARTICLEBURST
	message_begin(MSG_BROADCAST,SVC_TEMPENTITY)
	write_byte(122) // very similar to lavasplash.
	write_coord(xy[0]) // pos
	write_coord(xy[1])  
	write_coord(xy[2]) 
	write_short (width)
	write_byte (color) // (particle color)
	write_byte (40) // (duration * 10) (will be randomized a bit)
	message_end()

	return PLUGIN_HANDLED
}

public amx_fw_all(id,level,cid){
	if (!cmd_access(id,level,cid,2))
		return PLUGIN_HANDLED

	if( (id != 0) && (!(get_user_flags(id) & ADMIN_LEVEL_B)) && (bigshow[id] > 0) ){
		console_print(id,"[AMX] You dont have enough access to do this command more than once")
		return PLUGIN_HANDLED
	}

	bigshow[id] += 1
	remove_task(6341)
	remove_task(6342)
	remove_task(6343)
	remove_task(6344)
	new arg1[8],iarg1
	read_argv(1,arg1,7)
	iarg1 = strtonum(arg1)
	if(iarg1 > 300)
		iarg1 = 300
	if(iarg1 < 1){
		client_print(0,print_chat,"[AMX]  Awww, no more fireworks.")
	}else{
		if(!(get_user_flags(id) & ADMIN_LEVEL_B))
			iarg1 = 20
		client_print(0,print_chat,"[AMX]  Oooooo! Ahhhhh!  Fireworks!")
		set_task(2.1,"fw_garbage",6341,"",0,"a",iarg1/2)
		set_task(1.6,"fw_tara",6342,"",0,"a",iarg1/2)
		set_task(2.3,"fw_ooo",6343,"",0,"a",iarg1/2)
		set_task(2.0,"fw_spr",6344,"",0,"a",iarg1/2)
	}
	return PLUGIN_HANDLED
}

public adjust_range(){
	new lowest
	new players[32], inum
	get_players(players,inum,"ac")
	for(new i = 0 ;i < inum; ++i){
		new origin[3]
		get_user_origin(players[i],origin)
		if(lowest > origin[2])
			lowest = origin[2]
	}
	if(mapadjust > lowest)
		mapadjust = lowest

	return PLUGIN_CONTINUE
}
public amx_fw_lcadj(id,level,cid){
	if (!cmd_access(id,level,cid,1))
		return PLUGIN_HANDLED
	if(lowceiling == false){
		lowceiling = true
		console_print(id,"[AMX] You have set low ceiling for the glowsprite fireworks to increase visability")
	}else{
		lowceiling = false
		console_print(id,"[AMX] You returned glowsprite fireworks to normal ceiling")
	}

	return PLUGIN_HANDLED
}

public amx_fw_sound(id,level,cid){
	if (!cmd_access(id,level,cid,1))
		return PLUGIN_HANDLED
	if(get_cvar_num("amx_fireworks_noise") == 1){
		set_cvar_string("amx_fireworks_noise","0")
		console_print(id,"[AMX] You turned fireworks sound OFF")
	}else{
		set_cvar_string("amx_fireworks_noise","1")
		console_print(id,"[AMX] You turned fireworks sound ON")
	}

	return PLUGIN_HANDLED
}

public set_fw_maprule(){ 
	new ThisMap[32]
	get_mapname(ThisMap, 31)
	if(file_exists("addons/amx/fireworks.ini") == 1){
		new line = 1
		new data[192]
		new stextsize = 0
		new map[32]
		while((line=read_file("addons/amx/fireworks.ini",line,data,191,stextsize))!=0){
			parse(data,map,31)
			if(equali(map,ThisMap)){
				server_cmd("amx_fireworks_lc")			
			}
		}
	}else{
		write_file("addons/amx/fireworks.ini","If a map has a low cieling like de_dust we want to set this to make sprites more visable.",-1)
		write_file("addons/amx/fireworks.ini","de_dust",-1) 
		write_file("addons/amx/fireworks.ini","de_dust2",-1) 
		write_file("addons/amx/fireworks.ini","de_dust1a",-1) 
		write_file("addons/amx/fireworks.ini","cs_office",-1)

		set_task(3.0,"set_fw_maprule") 
	}
	return PLUGIN_HANDLED
}

public restore_fw_sound(){
	set_cvar_string("amx_fireworks_noise","1")
	return PLUGIN_CONTINUE
}

public client_connect(id){
	bigshow[id] = 0
	return PLUGIN_CONTINUE
}

public client_disconnect(id){
	bigshow[id] = 0
	return PLUGIN_CONTINUE
}

public round_start(){
	DoOnce = 0
	antispam2 +=1
	if(antispam2 > 2){
		antispam = 0
		antispam2 = 0
	}
	antispam3 = 0
	return PLUGIN_CONTINUE
}

public round_end(){
	if(get_cvar_num("amx_fireworks_roundend") == 0)
		return PLUGIN_CONTINUE
	if(DoOnce == 0){
		if(get_cvar_num("amx_fireworks_roundend") == 1){	
			if(get_cvar_num("amx_fireworks_noise") == 1){
				set_cvar_string("amx_fireworks_noise","0")
				set_task(10.0,"restore_fw_sound")
			}
		}
		DoOnce = 1
		remove_task(6341)
		remove_task(6342)
		remove_task(6343)
		remove_task(6344)
		set_task(2.1,"fw_garbage",6341,"",0,"a",3)
		set_task(1.6,"fw_tara",6342,"",0,"a",3)
		set_task(2.3,"fw_ooo",6343,"",0,"a",3)
		set_task(2.0,"fw_spr",6344,"",0,"a",3)
	}
	return PLUGIN_CONTINUE
}

public plugin_precache(){ 
	new mod_name[32]
	get_modname(mod_name,31)
	dod_running = equal(mod_name,"dod") ? true : false
	if(dod_running)
		smoke = precache_model("sprites/smoke.spr") 		
 	else
		smoke = precache_model("sprites/steam1.spr")
	mflash = precache_model("sprites/muzzleflash.spr") 
	flare3 = precache_model("sprites/flare3.spr")
	flare6 = precache_model("sprites/flare6.spr")
	blueflare2 = precache_model("sprites/blueflare2.spr")
	redflare2 = precache_model("sprites/redflare2.spr")
	garbgibs = precache_model("models/garbagegibs.mdl")
	l_tube = precache_model("models/w_flare.mdl")
	precache_sound("weapons/grenade_hit2.wav")
	return PLUGIN_CONTINUE 
}

public plugin_init(){
	register_plugin("Fireworks Display","0.9","EJL")
	register_concmd("amx_fireworks1","amx_fw_g",ADMIN_LEVEL_H,"<time or 0 to stop> Sets debris model fireworks off at random points on map ")
	register_concmd("amx_fireworks2","amx_fw_tara",ADMIN_LEVEL_H,"<time or 0 to stop> Sets quake1 tar expolsion fireworks off at random points on map ")
	register_concmd("amx_fireworks3","amx_fw_ooo",ADMIN_LEVEL_H,"<time or 0 to stop> Sets big particle burst fireworks off at random points on map ")
	register_concmd("amx_fireworks4","amx_fw_rc",ADMIN_LEVEL_H,"<time or 0 to stop> Roman candle -- not done yet, sorry.")
	register_concmd("amx_fireworks5","amx_fw_spr",ADMIN_LEVEL_H,"<time or 0 to stop> Sets glow sprite fireworks off at random points on map ")
	register_concmd("amx_fireworks","amx_fw_all",ADMIN_LEVEL_H,"<time or 0 to stop> Finale style! Sets four types fireworks off at random points on map.")
	register_concmd("amx_fireworks_set","amx_fw_set",ADMIN_LEVEL_H,"<type 0-5 or cancell> <height> <quantity> <delay> : Sets or cancels local fireworks")
	register_concmd("amx_fireworks_lc","amx_fw_lcadj",ADMIN_LEVEL_H,": sets whether type 1 and 5 fireworks explode low on maps with a low ceiling")
	register_concmd("amx_fireworks_sound","amx_fw_sound",ADMIN_LEVEL_H,": toggles fireworks noise on and off.")

	register_cvar("amx_luds_fireworks","v1.0",FCVAR_SERVER )
	register_cvar("amx_fireworks_noise","1")
	set_cvar_string("amx_fireworks_noise","1")
	register_cvar("amx_fireworks_roundend","0")
	set_cvar_string("amx_fireworks_roundend","0")

	register_event("SendAudio","round_end","a","2=%!MRAD_terwin","2=%!MRAD_ctwin","2=%!MRAD_rounddraw")
	register_event("TextMsg","round_end","a","2&#Game_C","2&#Game_w")
	register_event("SendAudio","round_start","bc","2=%!MRAD_GO","2=%!MRAD_MOVEOUT","2=%!MRAD_LETSGO","2=%!MRAD_LOCKNLOAD")	

	set_task(47.0,"adjust_range",5132,"",0,"a",9999)
	set_task(1.9,"set_fw_maprule")

	return PLUGIN_CONTINUE
}
