/***************************************************************************
 *   amx_noswear.sma             Version 1.6               Date: FEB/10/2004
 *
 *   RS Swear Filter + Punishment (MySQL/STEAM Support)
 *
 *   By:      Rob Secord
 *   Alias:   xeroblood (aka; sufferer)			   xeroblood@msn.com
 *
 *   Credits: Util, Knocker, poosmith
 *
 *   Tested / Developed On:
 *      CS 1.5 - AMX 0.9.6j
 *      CS 1.6 - AMX 0.9.8
 *      MySQL v4.0.16-nt (Driver: v03.51) 
 *
 ***************************************************************************
 *  Admin Commands:
 *     amx_swearmode <on|off>   - In-game admin command to Turn swearMode
 *                                on or off. (removes any current gags)
 *     amx_swearmenu            - Toggles Swear Menu To Be Attached to YOU
 *                                If Swear Menu is attached to an Admin, only
 *                                that admin will see the Swear Menu when a
 *                                player swears too many times.  If SwearMenu
 *                                is not attached to an admin, then Gag mode
 *                                is the default!
 *     amx_namecheck            - Runs a Swear Filter On All Clients' Username
 *                                Should be called if CVAR sv_checknames is
 *                                changed to 1 In-Game.
 *
 *  Server CVARS:
 *     sv_noswear <1|0>        - Turns Swear Mod On/Off
 *     sv_sweartimes #         - The number of Swears Allowed
 *                               If Set to 0, Punishment Is OFF
 *     sv_usefilters <1|0>     - Uses Word Filters (fu.ck) if set to 1
 *     sv_whitelist <1|0>      - Uses Whitelist for GOOD words, if set to 1
 *     sv_cashloss #           - The Amount of Cash Lost as Punishment
 *     sv_gagtime #            - # of Sec. player will be Gagged
 *     sv_bantime #            - # of Min. player will be Banned
 *     sv_bantype <1|0>        - BanType by ID(0) or IP(1)
 *     sv_censorwords <1|0>    - Display/Censor Swear Word
 *                               (if set to 1, swear word = ****)
 *     sv_censorchar "*"       - The Character used to censor Swear Words
 *     sv_checknames <1|0>     - Checks player names for Swear words if 1
 *     sv_rename_mode <1|2|3>  - If players name is Swear Word, renames to
 *                                1 - WonID / AuthID
 *                                2 - IP Address
 *                                3 - "Potty-Mouth #"
 *     sv_swearhelp <1|0>      - If set to 1, players can say /swearwords
 *                               to see the list of offensive words
 *
 **************************************************************************
 *  Note:
 *     swearwords.ini  must be put in $MODDIR$\addons\amx\swearwords.ini
 *     with one swear word per line!
 *     Same with goodwords.ini if using whitelist (sv_whitelist 1)
 *     
 *     If longest swear word is > 24 characters, change MAX_WORD_LENGTH
 *     to reflect longest word.
 *     If you have more that 40 words, change MAX_SWEAR_WORDS to reflect
 *     the number of words in file!
 *
 *     If you use MySQL, run the file wordtables.sql included in ZIP, or
 *     create these tables, and add values to them:
 *
 * Create Table If Not Exists RS_SwearWords
 *   ( Word varchar(24) NOT NULL default '', Primary Key (Word) )
 *   TYPE=MyISAM;
 *
 * Create Table If Not Exists RS_GoodWords
 *   ( Word varchar(24) NOT NULL default '', Primary Key (Word) )
 *   TYPE=MyISAM;
 *
 **************************************************************************
 *   Credits:
 *      As Always, Much of the credit goes to The People and Tutorials at
 *      http://amxmod.net/forums for lots of information, and good advice!!
 *
 *   Plugin Topic:
 *      http://amxmod.net/forums/viewtopic.php?t=27801
 *
 *   Ideas/Suggestions:
 *      Util, Knocker (see Updates below)
 *
 *   Linux Compile:
 *      Knocker
 *
 *   Bug Fix (Case-Sensitivity):
 *      poosmith
 *
 **************************************************************************
 *
 *  Updates To Version 1.6:
 *    - Fixed support for non-MySql admins (requires recompile)
 *      (Removed CVAR sv_usemysql -- Defined USE_MYSQL instead)
 *    - Compiled on Linux SUSE 9.0 by Knocker!!
 *
 *  Updates To Version 1.5:
 *    - Checks Names for Swear Words if CVAR sv_checknames is set to 1
 *    - Renames Offensive names to WONID, IP or "Potty-Mouth #". Change by
 *      setting CVAR sv_rename_mode to 1, 2 or 3 respectively.
 *
 *    - Uses MySQL or FILE for Swear Word List. To Use MySQL, set CVAR
 *      sv_usemysql to 1, default is 0 (Read From File)
 *
 *    - Added a Whitelist for Good Words (goodwords.ini, or MySQL Table)
 *
 *    - Uses Word Filters (fu.ck) If CVAR sv_usefilters is set to 1
 *    - Added CVAR sv_swearexact to Test against whole words or substrings
 *      (This should help protect ppl who say FUN when FU is a Swear Word)
 *      (Idea: Knocker)
 *
 *    - Added A New Menu Punishment! Remove Cash From Player, Amount of
 *      cash removed is in CVAR sv_cashloss
 *
 *    - Added a CVAR for the Special Character used for Censoring Words!!
 *      (instead of ****, see whatever char you want!!)
 *      (Idea: Knocker)
 *
 *    - Changed: if sv_sweartimes is set to 0, Punishment Is Turned OFF!
 *
 *    - Added a Help MOTD for players who say /swearwords.  Admins may
 *      disable this option with CVAR sv_swearhelp
 *
 *  Updates To Version 1.4: (Credits: Util for the Ideas!)
 *    - Reads Swear Words From File (addons/amx/swearwords.ini)
 *    - Added Ban Option (Ban Perm / # Minutes)
 *       (# Minutes is a CVAR: amx_bantime #)
 *       (Ban By WONID or IP)
 *    - Added Swear Word To Menu
 *
 *  Updates To Version 1.4: (Credits: Knocker for the Ideas!)
 *    - Added Notice to Users When They Swear
 *    - Added CVAR for displaying/censoring Swear Word
 *    - Added Word Filter to catch words like:  fu.ck  or   f*u.C,k   etc...
 *
 *  Versions prior to 1.4 unreleased...
 *
 **************************************************************************/

#define USE_MYSQL 0    //  Set to 0 and recompile if you dont use MySQL!!

#include <amxmod>
#include <amxmisc>
#include <fun>

#if USE_MYSQL
#include <mysql>
#endif

#define REQD_LEVEL ADMIN_BAN	// Change to your Desired Admin Access Level

#define MAX_WORD_LENGTH 24   // Change if longest Word is > 24 characters
#define MAX_SWEAR_WORDS 40   // Maximum number of BLACKLIST Words (Change if more needed)
#define MAX_GOOD_WORDS  40   // Maximum number of WHITELIST Words (Change if more needed)
#define NUM_FILTERS     18   // Change if you add filter elements for Swear Words (g_szFilter)

#define MAX_PLAYERS     32   // Don't Change
#define MAX_SPEECH_LEN  256  // Don't Change

#define MAX_NAME_LENGTH 32

new g_szSwearWords[MAX_SWEAR_WORDS][MAX_WORD_LENGTH]
new g_szGoodWords[MAX_SWEAR_WORDS][MAX_WORD_LENGTH]

new g_szBadWord[MAX_PLAYERS][MAX_WORD_LENGTH]

#if !USE_MYSQL
new g_szBadWordFile[64]
new g_szGoodWordFile[64]
#endif

new g_nSwearTimes[MAX_PLAYERS] = 0
new g_nSwearNames[MAX_PLAYERS] = 0
new g_nUserGagged[MAX_PLAYERS] = 0
new g_nUserSwore[MAX_PLAYERS] = 0
new g_nCurrClientID = 0
new g_nShowAdmin = 0
new g_nOffenderNum = 0

new g_szUserSpeech[MAX_PLAYERS][MAX_SPEECH_LEN]
new g_szUserCmd[MAX_PLAYERS][32]

new g_szFilter[NUM_FILTERS][2] = { "!", "@", "#", "$", "%", "&", "*", ",", ".", ":", ";", "~", "`", "'", "/", "-", "+", "_" }


public plugin_init()
{
	register_plugin( "RS Swear Filter", "1.6", "xeroblood" )
	
	register_cvar( "sv_noswear",     "1" )
	register_cvar( "sv_sweartimes",  "5" )
	register_cvar( "sv_usefilters",  "1" )
	register_cvar( "sv_whitelist",   "1" )
	register_cvar( "sv_cashloss",    "1000" )
	register_cvar( "sv_gagtime",     "120" )
	register_cvar( "sv_bantime",     "1440" )
	register_cvar( "sv_bantype",     "0" )
	register_cvar( "sv_censorwords", "1" )
	register_cvar( "sv_censorchar",  "*" )
	register_cvar( "sv_checknames",  "1" )
	register_cvar( "sv_rename_mode", "3" )
	register_cvar( "sv_swearhelp",   "1" )

#if USE_MYSQL
	if( !cvar_exists( "amx_mysql_host" ) )
        	register_cvar( "amx_mysql_host", "127.0.0.1" )   //  Note:
        if( !cvar_exists( "amx_mysql_user" ) )
        	register_cvar( "amx_mysql_user", "root" )        //  If your MySQL Info is
        if( !cvar_exists( "amx_mysql_pass" ) )
        	register_cvar( "amx_mysql_pass", "" )            //  different, you should
        if( !cvar_exists( "amx_mysql_db" ) )
        	register_cvar( "amx_mysql_db", "amx" )           //  edit the file mysql.cfg
        server_cmd("exec addons/amx/mysql.cfg;")                 //  in your addons/amx folder
#endif

#if !defined NO_STEAM
	register_menucmd( register_menuid("Swear Menu"), 1023, "SwearMenuCommand" )
#else
	register_menucmd( register_menuid("\rSwear Menu"), 1023, "SwearMenuCommand" )
#endif

	register_clcmd( "amx_swearmode", "ToggleSwearMod",    REQD_LEVEL, "< on | off > -- Toggles Swear Mod On or Off" )
	register_clcmd( "amx_swearmenu", "AttachMenuToAdmin", REQD_LEVEL, "Toggle Switch For Admins To See Menu" )
	register_clcmd( "amx_namecheck", "RunNameCheck",      REQD_LEVEL, "Runs A Name Check On All Clients" )	

	register_clcmd( "say",      "ClCmd_CheckSay" )
	register_clcmd( "say_team", "ClCmd_CheckSay" )
	register_clcmd( "say /swearwords", "SwearWordsHelp" )
}

public plugin_cfg()
{
	set_task( 1.0, "LoadSwearWords", 3500 )
}

#if !defined NO_STEAM
public client_authorized( id )
#else
public client_connect( id )
#endif
{
	if( get_cvar_num( "sv_noswear" ) )
	{
		client_cmd( id, "echo ^"Server Is Running RS Swear Filter Plugin! Do Not Swear!^"")
		
		if( get_cvar_num( "sv_swearhelp" ) == 1 )
			client_cmd( id, "echo ^"To See the List Of Swear Words, Say /swearwords^"")

		ClCmd_CheckName( id )
	}
}

public client_infochanged( id ) 
{
	#if !defined NO_STEAM
		if ( !is_user_connected( id ) ) return PLUGIN_CONTINUE
	#endif
	if( PlayerHasImmunity( id ) ) return PLUGIN_CONTINUE

	new szNewName[MAX_NAME_LENGTH]
	new szOldName[MAX_NAME_LENGTH]

	get_user_info( id, "name", szNewName, MAX_NAME_LENGTH-1 )
	get_user_name( id, szOldName, MAX_NAME_LENGTH-1 ) 

	if( !equal( szNewName, szOldName ) )
		return ClCmd_CheckName( id )

	return PLUGIN_CONTINUE
} 

public client_disconnect( id )
{
	g_nSwearTimes[id-1] = 0
	g_nUserGagged[id-1] = 0
	if( g_nShowAdmin == id )
		g_nShowAdmin = 0
		
	remove_task( 2000 + id )
	return PLUGIN_CONTINUE
}

public ToggleSwearMod( id, lvl, cid )
{
	if( !cmd_access( id, lvl, cid, 2 ) ) 
		return PLUGIN_HANDLED

	new szArg[32]
	read_argv( 1, szArg, 31 ) 
	new nOnOff = ( equali( szArg, "on" ) || equali( szArg, "1" ) ) ? 1 : 0

	set_cvar_num( "sv_noswear", nOnOff )
	console_print( id, "[AMX] Swear Mode Set To %i!", nOnOff )

	if( nOnOff ) return PLUGIN_HANDLED

	for( new i = 0; i < MAX_PLAYERS; i++ )
	{
		g_nSwearTimes[i] = 0
		g_nUserGagged[i] = 0
		if( task_exists( 2000 + i ) )
			remove_task( 2000 + i )
	}

	return PLUGIN_HANDLED
}

public AttachMenuToAdmin( id, lvl, cid )
{
	if( !cmd_access( id, lvl, cid, 1 ) )
		return PLUGIN_HANDLED

	if( g_nShowAdmin )
	{
		if( g_nShowAdmin != id )
		{
			new szAdminName[32]
			get_user_name( g_nShowAdmin, szAdminName, 31 )
			console_print( id, "ADMIN: %s Already has the Swear Menu attached!", szAdminName )
		}else
		{
			g_nShowAdmin = 0
			console_print( id, "You Have Unattached The Swear Menu!" )
		}
	}else
	{
		g_nShowAdmin = id
		console_print( id, "The Swear Menu is now attached to you!" )
	}
	return PLUGIN_HANDLED
}


public ShowMenuToAdmin( aID, vID )
{
	new szUser[32]
	new szMenuBody[512]
	new nKeys = (1<<0|1<<1|1<<2|1<<3|1<<4|1<<9)
	new nCashLoss = get_cvar_num( "sv_cashloss" )

	g_nCurrClientID = vID

	get_user_name( vID, szUser, 31 )
	format( szMenuBody, 511, "\rSwear Menu^n^n\w%s (%s)\w^n^n1. \yGag\w^n2. \yKick\w^n3. \yRemove Cash ($%d)\w^n4. \yBan Temporary\w^n5. \yBan Permanent\w^n^n0. Take No Action", szUser, g_szBadWord[vID-1], nCashLoss )
	
	show_menu( aID, nKeys, szMenuBody )
}

public SwearMenuCommand( id, key )
{
	switch( key )
	{
		case 0: ActionGag( g_nCurrClientID )
		case 1: ActionKick( g_nCurrClientID )
		case 2: ActionCashLoss( g_nCurrClientID )
		case 3: ActionBanTemp( g_nCurrClientID )
		case 4: ActionBanPerm( g_nCurrClientID )
	}
	return PLUGIN_HANDLED
}

public ClCmd_CheckSay( id )
{	
	if( !get_cvar_num( "sv_noswear" ) ) return PLUGIN_CONTINUE
	if( is_user_bot( id ) ) return PLUGIN_CONTINUE
	if( PlayerHasImmunity( id ) ) return PLUGIN_CONTINUE

	// Commented to test against self
	//if( g_nShowAdmin == id ) return PLUGIN_CONTINUE

	if( g_nUserSwore[id-1] )
	{
		g_nUserSwore[id-1] = 0
		return PLUGIN_CONTINUE
	}

	if( g_nUserGagged[id-1] )
	{
		client_print( id, print_chat, "You Have Been Gagged for Swearing...You Must Wait!" )
		if( get_cvar_num( "sv_swearhelp" ) == 1 )
			client_print( id, print_chat, "Say /swearwords for a list of Offensive Words!" )
		return PLUGIN_HANDLED
	}


	read_argv( 0, g_szUserCmd[id-1], 31 )
	read_args( g_szUserSpeech[id-1], MAX_SPEECH_LEN-1 )
	remove_quotes( g_szUserSpeech[id-1] )

	if( FilterText( id, g_szUserSpeech[id-1] ) )
	{
		if( get_cvar_num("sv_sweartimes") > 0 )
		{
			g_nSwearTimes[id-1]++
			g_nUserSwore[id-1] = 1
		
			if( g_nSwearTimes[id-1] >= get_cvar_num("sv_sweartimes") ) 
			{
				if( g_nShowAdmin )
					ShowMenuToAdmin( g_nShowAdmin, id )
				else
					ActionGag( id )
			}else
			{
				new x = (get_cvar_num("sv_sweartimes") - g_nSwearTimes[id-1] )
				client_print( id, print_chat, "STOP SWEARING!! %d More Time%s And YOU GET PUNISHED!", x, (x>1)?"s":"" )
			}
		}

		new aID[2]
		aID[0] = id
		// If user swears, run filter 0.8 seconds later to avoid flooding server
		// If you dont use anti-flood, set to 0.1
		set_task( 0.8, "RunSayFilter", (3000 + id), aID, 1 )

		return PLUGIN_HANDLED
	}

	return PLUGIN_CONTINUE
}

public ClCmd_CheckName( id )
{
	if( !get_cvar_num( "sv_noswear" ) ) return PLUGIN_CONTINUE
	if( !get_cvar_num( "sv_checknames" ) ) return PLUGIN_CONTINUE
	if( is_user_bot( id ) ) return PLUGIN_CONTINUE
	if( PlayerHasImmunity( id ) ) return PLUGIN_CONTINUE

	new szUserName[MAX_NAME_LENGTH]
	get_user_info( id, "name", szUserName, MAX_NAME_LENGTH-1 )

	if( FilterText( id, szUserName ) )
	{
		if( get_cvar_num("sv_sweartimes") > 0 )
		{
			g_nSwearNames[id-1]++
		
			if( g_nSwearNames[id-1] >= get_cvar_num("sv_sweartimes") ) 
			{	
				if( g_nShowAdmin )
					ShowMenuToAdmin( g_nShowAdmin, id )
				else
					ActionKick( id )
			}else
			{
				new x = (get_cvar_num("sv_sweartimes") - g_nSwearNames[id-1] )
				client_print( id, print_chat, "STOP NAME-SWEARING!! %d More Time%s And YOU GET PUNISHED!", x, (x>1)?"s":"" )
			}
		}

		new szNewName[MAX_NAME_LENGTH]
		switch( get_cvar_num( "sv_rename_mode" ) )
		{
			case 1:
			#if !defined NO_STEAM
				get_user_authid( id, szNewName, MAX_NAME_LENGTH-1 )
			#else
				format( szNewName, MAX_NAME_LENGTH-1, "%d", get_user_wonid(id) )
			#endif
			case 2: get_user_ip( id, szNewName, MAX_NAME_LENGTH-1, 0 )
			default: format( szNewName, MAX_NAME_LENGTH-1, "Potty-Mouth %d", ++g_nOffenderNum )
		}
		set_user_info( id, "name", szNewName )
		client_print( id, print_console, "Your Name Has Been Changed To: %s", szNewName )

		return PLUGIN_HANDLED
	}

	return PLUGIN_CONTINUE
}

public RunNameCheck( id, lvl, cid )
{
	if( !cmd_access( id, lvl, cid, 1 ) )
		return PLUGIN_HANDLED

	for( new i = 0; i < MAX_PLAYERS; i++ )
		ClCmd_CheckName( id )

	return PLUGIN_HANDLED
}

public RunSayFilter( p_ID[] )
{
	client_cmd( p_ID[0], "%s ^"%s^"", g_szUserCmd[p_ID[0]-1], g_szUserSpeech[p_ID[0]-1] )
	remove_task( 3000 + p_ID[0] )
}

public PlayerHasImmunity( id )
{
	return (get_user_flags( id ) & ADMIN_IMMUNITY) ? true : false
}

public ActionGag( id )
{
	new aCmdParm[2]
	aCmdParm[0] = id
	new Float:fGagTime = float(get_cvar_num("sv_gagtime"))
	set_task( fGagTime, "ActionUngag", (2000 + id), aCmdParm, 1 )
				
	new szActionMsg[64], szUserName[32]
	get_user_name( id, szUserName, 31 )
	format( szActionMsg, 63, "%s Has Just Been Gagged!", szUserName )
	AnnounceAction( szActionMsg )
				
	g_nUserGagged[id-1] = 1
	client_print( id, print_chat, "You Have Been Gagged for %i Seconds for Swearing!", get_cvar_num("sv_gagtime") )

}

public ActionUngag( p_aCmdParm[] )
{
	new szActionMsg[64], szUserName[32]

	g_nUserGagged[p_aCmdParm[0]-1] = 0
	client_print( p_aCmdParm[0], print_center, "You have been Ungagged!! Watch What You Say Next Time!!" )
	
	get_user_name( p_aCmdParm[0], szUserName, 31 )
	format( szActionMsg, 63, "%s Has Just Been Ungagged!", szUserName )
	AnnounceAction( szActionMsg )
	
	remove_task( 2000 + p_aCmdParm[0] )
}

public ActionKick( id )
{
	new szActionMsg[64], szUserName[32]
	get_user_name( id, szUserName, 31 )
	format( szActionMsg, 63, "%s Has Just Been Kicked For Swearing!", szUserName )
	AnnounceAction( szActionMsg )

	server_cmd( "kick #%d", get_user_userid(id) )
	console_print( id, "You Have Been Kicked For Swearing!" )
}

public ActionCashLoss( id )
{
	new nCash = get_user_money( id )
	new nCashLoss = get_cvar_num( "sv_cashloss" )

	if( nCash-nCashLoss > -1 )
		set_user_money( id, nCash-nCashLoss )
	else
		set_user_money( id, 0 )

	client_print( id, print_chat, "You Have Lost $%d As Punishment For Swearing!", nCashLoss )
}

public ActionBanTemp( id )
{
	new szActionMsg[64], szUserName[32], szUserID[32]
	get_user_name( id, szUserName, 31 )
	format( szActionMsg, 63, "%s Has Been Banned For Swearing For %d Minutes!", szUserName, get_cvar_num("sv_bantime") )
	AnnounceAction( szActionMsg )

	if( get_cvar_num( "sv_bantype" ) == 1 )
	{
		get_user_ip( id, szUserID, 31, 1 )
		server_cmd( "addip ^"%d^" ^"%s^";wait;writeip", get_cvar_num("sv_bantime"), szUserID )
	}else
	{
		get_user_authid( id, szUserID, 31 )
		server_cmd( "banid ^"%d^" ^"%s^" kick;wait;writeid", get_cvar_num("sv_bantime"), szUserID )
	}
	console_print( id, "You Have Been Banned For Swearing For %d Minutes!", get_cvar_num("sv_bantime") )
}

public ActionBanPerm( id )
{
	new szActionMsg[64], szUserName[32], szUserID[32]
	get_user_name( id, szUserName, 31 )
	format( szActionMsg, 63, "%s Has Been Banned For Swearing Permanently!", szUserName, get_cvar_num("sv_bantime") )
	AnnounceAction( szActionMsg )

	if( get_cvar_num( "sv_bantype" ) == 1 )
	{
		get_user_ip( id, szUserID, 31, 1 )
		server_cmd( "addip ^"%d^" ^"%s^";wait;writeip", 0, szUserID )
	}else
	{
		get_user_authid( id, szUserID, 31 )
		server_cmd( "banid ^"%d^" ^"%s^" kick;wait;writeid", 0, szUserID )
	}
	console_print( id, "You Have Been Permanently Banned For Swearing!" )
}

public LoadSwearWords()
{
	new i = 0	
	remove_task( 3500 )	

#if USE_MYSQL
	new cnMySql, errMySql[128]
	new szHost[64], szUser[32], szPass[32], szDatabase[32]
	get_cvar_string( "amx_mysql_host", szHost, 63)
	get_cvar_string( "amx_mysql_user", szUser, 31)
	get_cvar_string( "amx_mysql_pass", szPass, 31)
	get_cvar_string( "amx_mysql_db",   szDatabase, 31)

	cnMySql = mysql_connect( szHost, szUser, szPass, szDatabase, errMySql, 127)

	if( cnMySql < 1)
	{
		server_print( "RS Swear Filter - DB Error: '%s'", errMySql )
		return PLUGIN_HANDLED
	}

	// Get BLACKLIST Words...
	if( mysql_query( cnMySql, "Select Word From RS_SwearWords" ) < 1 )
	{
		mysql_error( cnMySql, errMySql, 127 )
		server_print( "RS Swear Filter - DB Error: '%s'", errMySql )
		return PLUGIN_HANDLED
	}
	while( mysql_nextrow( cnMySql ) > 0 )
		mysql_getfield( cnMySql, 1, g_szSwearWords[i++], MAX_WORD_LENGTH-1 )

	server_print( "RS Swear Filter: %d Bad Word%s Loaded From MySQL!", i, (i == 1) ? "" : "s" )
	if( get_cvar_num( "sv_whitelist" ) == 1 )
	{
		i = 0
		// Get WHITELIST Words...
		if( mysql_query( cnMySql, "Select Word From RS_GoodWords" ) < 1 )
		{
			mysql_error( cnMySql, errMySql, 127 )
			server_print( "RS Swear Filter - DB Error: '%s'", errMySql )
			return PLUGIN_HANDLED
		}
		while( mysql_nextrow( cnMySql ) > 0 )
			mysql_getfield( cnMySql, 1, g_szGoodWords[i++], MAX_WORD_LENGTH-1 )
		server_print( "RS Swear Filter: %d Good Word%s Loaded From MySQL!", i, (i == 1) ? "" : "s" )
	}

	mysql_close( cnMySql )
#else
	new szWord[MAX_WORD_LENGTH]
	build_path( g_szBadWordFile, 63, "$basedir/swearwords.ini" )
	build_path( g_szGoodWordFile, 63, "$basedir/goodwords.ini" )

	new l = 0, pos = 0

	// Get BLACKLIST Words...
	if( !file_exists(g_szBadWordFile) )
	{
		server_print( "RS Swear Filter Error: File Not Found '%s'", g_szBadWordFile )
		return PLUGIN_HANDLED
	}

	while( read_file( g_szBadWordFile, pos++, szWord, MAX_WORD_LENGTH-1, l ) )
	{
		if( szWord[0] == ';' || !l ) continue
		parse( szWord, szWord, MAX_WORD_LENGTH-1 )
		copy( g_szSwearWords[i++], MAX_WORD_LENGTH-1, szWord )
	}
	server_print( "RS Swear Filter: %d Bad Word%s Loaded From File!", i, (i == 1) ? "" : "s" )

	if( get_cvar_num( "sv_whitelist" ) == 1 )
	{
		i = 0, l = 0, pos = 0
		// Get WHITELIST Words...
		if( !file_exists(g_szGoodWordFile) )
		{
			server_print( "RS Swear Filter Error: File Not Found '%s'", g_szGoodWordFile )
			return PLUGIN_HANDLED
		}
		while( read_file( g_szGoodWordFile, pos++, szWord, MAX_WORD_LENGTH-1, l ) )
		{
			if( szWord[0] == ';' || !l ) continue
			parse( szWord, szWord, MAX_WORD_LENGTH-1 )
			copy( g_szGoodWords[i++], MAX_WORD_LENGTH-1, szWord )
		}
		server_print( "RS Swear Filter: %d Good Word%s Loaded From File!", i, (i == 1) ? "" : "s" )
	}
#endif
	return PLUGIN_CONTINUE
}

public AnnounceAction( p_szAction[] )
{
	new nActivity = get_cvar_num( "amx_show_activity" )
	
	if( nActivity )
	{
		new szMsg[128], nLen

		if( nActivity == 1 && g_nShowAdmin )
		{
			new szAdminName[32]
			get_user_name( g_nShowAdmin, szAdminName, 31 )
			nLen = format( szMsg, 127, "[Admin:%s] ", szAdminName )
		}
		else nLen = format( szMsg, 127, "[Admin] " )
		
		nLen = format( szMsg[nLen], (127-nLen), p_szAction )

		client_print( 0, print_chat, szMsg )
	}
}

public FilterText( id, szText[] )
{
	new i, j, nPos = 0, nLen = 0
	new nFound = 0, nSkip = 0
	new szTmp[64], szChar[2], szStr[MAX_WORD_LENGTH]

	if( get_cvar_num( "sv_usefilters" )  )
		for( i = 0; i < NUM_FILTERS; i++ )
			while( containi( szText, g_szFilter[i] ) > -1 )
				replace( szText, MAX_SPEECH_LEN-1, g_szFilter[i], "" )

	for( i = 0; i < MAX_SWEAR_WORDS; i++ )
	{
		if( !g_szSwearWords[i][0] ) break
		
		nPos = containi( szText, g_szSwearWords[i] )
		while( nPos > -1 )
		{
			nSkip = 0
			nLen = strlen(g_szSwearWords[i])
			if( get_cvar_num( "sv_whitelist" ) )
			{
				parse( szText[nPos], szTmp, 63 )
				if( !equali( szTmp, g_szSwearWords[i] ) )
				{
					for( j = 0; j < MAX_GOOD_WORDS; j++ )
					{
						if( !g_szGoodWords[j][0] ) break
						if( containi( szTmp, g_szGoodWords[j] ) > -1 )
						{
							nSkip = 1
							break
						}
					}
				}
			}
			if( !nSkip )
			{
				nFound = 1
				copy( g_szBadWord[id-1], MAX_WORD_LENGTH-1, g_szSwearWords[i] )
				if( get_cvar_num( "sv_censorwords" ) )
				{
					get_cvar_string( "sv_censorchar", szChar, 1 )
					setc( szStr, nLen, szChar[0] )
					strtolower( szText[nPos] )
					strtolower( g_szSwearWords[i] )
					replace( szText[nPos], MAX_SPEECH_LEN-nPos-1, g_szSwearWords[i], szStr )
				}
			}
			nPos = containi( szText[nPos+nLen], g_szSwearWords[i] )
		}
	}

	return nFound			
}

public SwearWordsHelp( id )
{
	if( !get_cvar_num( "sv_noswear" ) ) return PLUGIN_HANDLED
	if( !get_cvar_num( "sv_swearhelp" ) ) return PLUGIN_HANDLED

	new szMsg[64+MAX_WORD_LENGTH*MAX_SWEAR_WORDS], nLen
	new nSize = 63+MAX_WORD_LENGTH*MAX_SWEAR_WORDS

	nLen = format( szMsg, nSize, "Swear Words List:^n^n" )

	for( new i = 0; i < MAX_SWEAR_WORDS; i++ )
	{
		if( !g_szSwearWords[i][0] ) break
		nLen += format( szMsg[nLen], (nSize-nLen), "%s^n", g_szSwearWords[i] )
	}
	show_motd( id, szMsg, "RS Swear Filter + Punishment" )

	return PLUGIN_HANDLED
}
