// htb.sma
//
//  Homer Teams Balance

#include <amxmod>
#include <amxmisc>

new HTB_VERSION[] = "0.3"


//=================================================================
//     Configuration informations


// messages
new bool:HTB_TELLWTJ = true // tell about wtj tries
new bool:HTB_ANNOUNCE = true // announce team status at beginning of round
new bool:HTB_SAYOK = true // announce team status, if teams are alright
new bool:HTB_TYPESAY = true // use typesay


//=================================================================
//     global variables

#define MAX_PLAYERS 32

// timer to check team composition
#define FREQ_TEAM_CHECK 1.0

// team ids
#define UNASSIGNED	 	0
#define TS 						1
#define CTS					 	2
#define AUTO_TEAM 		5


// teams information
new teamScores[3] // score
new teamCounts[3] // number of players
new Float:teamSumKDR[3] // sum of Kills-Deaths Ratios
new teamName[3][] = { "Spectator", "TS", "CTS" }

// streak information
new lastWinner
new winStreaks // number of successive wins

// player arrays are 1-based, there is no player 0
new bool:isBeingTransfered[MAX_PLAYERS+1]
new playerTeam[MAX_PLAYERS+1]
new lastRoundSwitched[MAX_PLAYERS+1]
new wtjCount[MAX_PLAYERS+1]
new kills[MAX_PLAYERS+1]
new deaths[MAX_PLAYERS+1]
new Float:kdrs[MAX_PLAYERS+1]

new roundCounter
new lastRoundTB

//=================================================================
//     DEBUG Functions

#define DEBUG

new idDebug=-1
new steamidDebug[] = "STEAM_0:1:10628"
#define DBGbuffLen 1023
new DBGbuff[DBGbuffLen+1]

public check_idDebug(id)
{
	new userauthid[32]
	get_user_authid(id,userauthid,31)
	if (equal(userauthid,steamidDebug)) {
		idDebug=id
	}
}

public debug_print({Float,_}:... )
{
	if (idDebug!=-1) {
		format_args( DBGbuff, DBGbuffLen, 0 )
		// client_print(idDebug, print_chat, "[HTB-Debug] %s", DBGbuff)
		console_print(idDebug, "[HTB-Debug] %s", DBGbuff)
	}
}

public cmd_status(id,level,cid) { 
	if (!cmd_access(id,level,cid,1)) 
		return PLUGIN_HANDLED 
	
	console_print(id, "[HTB-Debug] idDebug is %d", idDebug)
	console_print(id, "[HTB-Debug] Teams composition: CT=%d T=%d", teamCounts[CTS], teamCounts[TS])
	console_print(id, "[HTB-Debug] Teams wins: CT=%d T=%d", teamScores[CTS], teamScores[TS])
	console_print(id, "[HTB-Debug] Last winner: %s with %d streak", teamName[lastWinner], winStreaks)
	
	console_print(id, "[HTB-Debug] Teams sumKDR: CT=%.2f T=%.2f", teamSumKDR[CTS], teamSumKDR[TS])
	if ((teamCounts[CTS]>0)&&(teamCounts[TS]>0))
		console_print(id, "[HTB-Debug] Teams can win: CT=%.2f T=%.2f", 
			teamSumKDR[CTS]/teamCounts[TS], teamSumKDR[TS]/teamCounts[CTS])

	new name[32], inum
	new players[MAX_PLAYERS]
	get_players(players, inum)
	for (new a = 0; a <inum; a++) 
	{
		new i=players[a]
		get_user_name(i, name, 31)
		console_print(id, "[HTB-Debug] player %2d: %s [%s] kdr=%d / %d = %.2f", i, name, teamName[playerTeam[i]], 
			kills[i], deaths[i], kdrs[i])
	}
			
	return PLUGIN_CONTINUE
}

public cmd_switchme(id,level,cid) {
	if (!cmd_access(id,level,cid,1))
		return PLUGIN_HANDLED

	console_print(id, "[HTB-Debug] testing team switch..")
	new parm[2]
	parm[0] = id
	parm[1] = 0

	if (get_user_team(id)==CTS)
		parm[1] = 1
	else if (get_user_team(id)==TS)
		parm[1] = 2

	if (parm[1]!=0) {
		new team = parm[1]
		
		//---- Technique AdminMod : marche pas toujours
		// client_cmd(id, "chooseteam; menu_select %d", parm[1])

		// vire les vgui_menus
		set_user_info(id, "_vgui_menus", "0")

		//---- Technique AMX plmenu.sma
		engclient_cmd(id,"chooseteam")
		engclient_cmd(id, "menuselect", team==1 ?  "1" : "2" )
		engclient_cmd(id, "menuselect", "5")
		client_cmd(id,"slot1")

	}

	set_task(2.0, "simple_check", 0, parm, 2)

	return PLUGIN_CONTINUE
}

public simple_check(parm[2])
{
	if (get_user_team(parm[0])==parm[1])
		console_print(parm[0], "[HTB-Debug] team switch worked!")
	else
		console_print(parm[0], "[HTB-Debug] team failed!")
} 

//=================================================================
//     Utils functions

Float:fdivWorkaround(Float:nom, Float:denom){
	if ( denom == 0.0) return nom
	return fabs(nom / denom)
}

displayMessage( d, r, g, b, a, {Float,_}:... ){
  new msg[192]
  format_args( msg, 191, 5 )
  if ( a && HTB_TYPESAY ){
    set_hudmessage(r, g, b, 0.05, 0.55, 0, 6.0, float(d) , 0.5, 0.15, 2)
    show_hudmessage( 0, msg )
  }
  client_print( 0 , print_chat , msg )
  server_print( msg )
}

//=================================================================
//     Initialization

public plugin_init(){	
	register_plugin("Team Balancer",HTB_VERSION,"Homer")
	register_cvar("amx_htb_version",HTB_VERSION,FCVAR_SERVER|FCVAR_EXTDLL|FCVAR_UNLOGGED|FCVAR_SPONLY)
	//--- round events
	register_event("SendAudio","round_end","a","2=%!MRAD_terwin","2=%!MRAD_ctwin","2=%!MRAD_rounddraw") // Round End
	register_event("RoundTime", "new_round", "bc") // Round Time
	register_event("TextMsg","game_restart","a","1=4","2&#Game_C","2&#Game_w") // Game restart
	//--- death events
	register_event("DeathMsg","death_msg","a") // Kill
	//--- score events
	register_event("TeamScore","team_score","a") // Team Score
	
	//--- team composition check
	set_task(FREQ_TEAM_CHECK, "check_teamcompos")

#if defined DEBUG
	//--- debugging functions
	register_concmd("amx_htbstatus","cmd_status",ADMIN_CFG,": get status")  
	register_concmd("amx_switchme","cmd_switchme",ADMIN_CFG,": test self switching")  
	
#endif
		
	//-- remove default team balance
	set_cvar_num("mp_limitteams", 10)
	set_cvar_num("mp_autoteambalance", 0)
	
	return PLUGIN_CONTINUE
}

//=================================================================
//     Rounds gestion
	
// call on restart
public game_restart(){
	roundCounter = 0
	lastRoundTB = 0
	teamScores[0] = teamScores[1] = teamScores[2] = 0
	lastWinner = UNASSIGNED
	winStreaks = 0
	for (new i = 1; i <= MAX_PLAYERS; ++i){
		kills[i] = 0
		deaths[i] = 0
		wtjCount[i] = 0
		lastRoundSwitched[i] = -999
	}
}

// call on round start
public new_round() {
	if ( floatround(get_cvar_float("mp_roundtime") * 60.0) != read_data(1) ) return
	++roundCounter

	// compute score
	init_kdrs()
	new Float:sCT = (teamCounts[TS] ? teamSumKDR[CTS]/teamCounts[TS] : teamSumKDR[CTS])
	new Float:sT = (teamCounts[CTS] ? teamSumKDR[TS]/teamCounts[CTS] : teamSumKDR[TS])
	
	new msg[128]	
	format(msg, 127, "Round %d - CTS %d TS %d^n balance: %.2f - %.2f", roundCounter, teamScores[CTS], teamScores[TS], sCT, sT)
	set_hudmessage(255, 255, 255, 0.45, 0.35, 0, 6.0, 5.0 , 0.5, 0.15, 2)
	show_hudmessage( 0, msg )
}

// call on end of round
public round_end(){
	new param[12]
	read_data(2,param,8)
	if (param[7]=='c') {//%!MRAD_ctwin
		if (lastWinner==CTS)
			winStreaks++
		else {
			lastWinner=CTS;
			winStreaks=1;
		}
	}
	else if (param[7]=='t') {//%!MRAD_terwin
		if (lastWinner==TS)
			winStreaks++
		else {
			lastWinner=TS;
			winStreaks=1;
		}
	}
	else
		return // %!MRAD_rounddraw (both teams have left the game)
	// do some team balance here
	set_task(1.0,"round_end_tb")
}

//=================================================================
//     Deaths gestion

public death_msg(){
	new iWinner = read_data(1)
	new iLoser = read_data(2)
	if ( iWinner < 1 || iWinner > MAX_PLAYERS || iLoser < 1 || iLoser > MAX_PLAYERS ) 
		return
	// if ( playerTeam[iWinner] == playerTeam[iLoser] )
	if ( get_user_team(iWinner) == get_user_team(iLoser) )
		return // no TKS!!!
	kills[iWinner]++
	deaths[iLoser]++
}

//=================================================================
//     Score Victory gestion

public team_score(){
	new arg[2]
	read_data(1,arg,1)
	teamScores[ ( arg[0] == 'T' ) ? TS : CTS ] = read_data(2)
}

//=================================================================
//     Players connection gestion

// public client_connect(id){
public client_authorized(id){
#if defined DEBUG
	check_idDebug(id)
#endif	
	kills[id] = 0
	deaths[id] = 0
	kdrs[id] = 1.0
	isBeingTransfered[id] = false
	playerTeam[id] = UNASSIGNED
	teamCounts[UNASSIGNED]++
	lastRoundSwitched[id] = -999
	wtjCount[id] = 0

	return PLUGIN_CONTINUE
}

public client_disconnect(id) {
#if defined DEBUG
	if (id==idDebug)
		idDebug = -1
#endif
	kills[id] = 0
	deaths[id] = 0
	kdrs[id] = 1.0
	isBeingTransfered[id] = false
	playerTeam[id] = UNASSIGNED
	lastRoundSwitched[id] = -999
	wtjCount[id] = 0
	
	new msg[128], name[32]
	get_user_name(id, name, 31)	
	format(msg, 127, "%s leaved the server!", name)
	set_hudmessage(0, 255, 0, 0.15, 0.65, 0, 6.0, 5.0 , 0.5, 0.15, 2)
	show_hudmessage( 0, msg )

	// redundant team size check
	set_teams()
	
	return PLUGIN_CONTINUE
}

//=================================================================
//     Teams composition gestion

public set_teams()
{
#if defined DEBUG
	debug_print("setting team composition")
#endif		
	teamCounts[UNASSIGNED] = 0
	teamCounts[CTS] = 0
	teamCounts[TS] = 0
	
	for (new i = 1; i <= MAX_PLAYERS; ++i) {
		playerTeam[i] = UNASSIGNED
	}
	new inum
	new players[MAX_PLAYERS]
	get_players(players, inum)
	for (new a = 0; a <inum; a++) 
	{
		new i=players[a]
		
		playerTeam[i] = get_user_team(i)
		++teamCounts[playerTeam[i]]
#if defined DEBUG
		new name[32]
		get_user_name(i, name, 31)
		// debug_print("%2d  %-16.15s %d", i, name, playerTeam[i])
#endif
	}
#if defined DEBUG
	debug_print("found %d CTS and %d TS", teamCounts[CTS], teamCounts[TS])
#endif		
}

public check_teamcompos(parm[])
{
#if defined DEBUG
	// debug_print("checking team composition")
#endif		
	//	for (new i = 1; i <= MAX_PLAYERS; ++i) {
	new inum
	new players[MAX_PLAYERS]
	get_players(players, inum)
	for (new a = 0; a <inum; a++) 
	{
		new i = players[a]
		new t = get_user_team(i)
		if ((playerTeam[i]==UNASSIGNED)&&(playerTeam[i]!=t)) {
			new msg[128], name[32]
			get_user_name(i, name, 31)	
			format(msg, 127, "%s entered the game as %s!", name, teamName[t])
			set_hudmessage(0, 0, 255, 0.15, 0.65, 0, 6.0, 5.0 , 0.5, 0.15, 2)
			show_hudmessage( 0, msg )
		}
		if (playerTeam[i] != t)
		{ // player has changed team
#if defined DEBUG
			new name[32]
			get_user_name(i, name, 31)
			debug_print("Player(%d): %s changed team from %d to %d", i, name, playerTeam[i], t)
#endif		
			// update team status
			++teamCounts[t]
			--teamCounts[playerTeam[i]]
			playerTeam[i] = t
		}
	}
	// will check again later
	set_task(FREQ_TEAM_CHECK, "check_teamcompos")
}

//=================================================================
//     Team transfert gestion

public transferPlayer(id, team)
{
	new parm[4]
	parm[0] = id
	parm[1] = team
	parm[2] = roundCounter
	parm[3] = 0 // number of tries
	
	isBeingTransfered[id] = true
	engclient_cmd(id,"chooseteam")
	engclient_cmd(id, "menuselect", team==1 ?  "1" : "2" )
	engclient_cmd(id, "menuselect", "5")
	client_cmd(id,"slot1")
	// set_task(0.5, "team_slot", 0, parm, 4)
	// set_task(1.0, "auto_skin", 0, parm, 4)
	set_task(3.0, "check_transfer", 0, parm, 4)
}

public team_slot(parm[4])
{
	client_cmd(parm[0],"slot%d",parm[1])
}

public auto_skin(parm[4])
{
	client_cmd(parm[0],"slot5") 
}

public check_transfer(parm[4])
{
	new name[32]
	get_user_name(parm[0], name, 31)

	if (get_user_team(parm[0]) == parm[1])
	{
		isBeingTransfered[parm[0]] = false		
		lastRoundSwitched[parm[0]] = roundCounter
		lastRoundTB = roundCounter
		client_print(0, print_chat, "[HTB] successfuly transfered %s to %s!", name, teamName[parm[1]])
		return
	}
	
	// check number of attempts	
	parm[3]++
	if (parm[3]>=2) {
		client_print(0, print_chat, "[HTB] did'nt managed to transfer %s to %s after 2 tries!", name, teamName[parm[1]])
		isBeingTransfered[parm[0]] = false				
		return
	}
	if ( (roundCounter<parm[2]) || (roundCounter-parm[2]>1) ) {
		client_print(0, print_chat, "[HTB] did'nt managed to transfer %s to %s in less than one round!", name, teamName[parm[1]])
		isBeingTransfered[parm[0]] = false		
		return
	}
	// retries to put player in team
	client_print(0, print_chat, "[HTB] tries to transfer %s to %s (%d)!", name, teamName[parm[1]], parm[3])
	engclient_cmd(parm[0],"chooseteam")
	new id = parm[0]
	new team = parm[1]
	engclient_cmd(id, "menuselect", team==1 ?  "1" : "2" )
	engclient_cmd(id, "menuselect", "5")
	client_cmd(id,"slot1")
	// set_task(0.5, "team_slot", 0, parm, 4)
	// set_task(0.1, "auto_skin", 0, parm, 4)
	set_task(3.0, "check_transfer", 0, parm, 4)	
}

//=================================================================
//     Score Skill gestion

public init_kdrs()
{
	teamSumKDR[UNASSIGNED] = 0.0
	teamSumKDR[CTS] = 0.0
	teamSumKDR[TS] = 0.0
	for (new i = 1; i <= MAX_PLAYERS; ++i) 
	{
		kdrs[i] = float(kills[i]+1) / float(deaths[i]+1)
		teamSumKDR[playerTeam[i]] += kdrs[i]
	}
}

public round_end_tb()
{
#if defined DEBUG
	debug_print("checking team balance")
#endif		
	set_teams()
	init_kdrs()

#if defined DEBUG	
	debug_print("Teams sumKDR: CT=%.2f T=%.2f", teamSumKDR[CTS], teamSumKDR[TS])
	if ((teamCounts[CTS]>0)&&(teamCounts[TS]>0))
		debug_print("Teams can win: CT=%.2f T=%.2f", 
			teamSumKDR[CTS]/teamCounts[TS], teamSumKDR[TS]/teamCounts[CTS])
#endif	

	new pCT, pT
	new nameCT[32], nameT[32]

	pCT = searchBestTransfer(CTS, 1)	
	if (pCT==0)
		pCT = searchBestTransfer(CTS, 0)	
	if (pCT==0)
		debug_print("No CTS transfert would improve balance!")
	else
	{
		get_user_name(pCT, nameCT, 31)
		debug_print("Transfering %s to the Terros would improve balance: CT=%.2f T=%.2f",
			nameCT, (teamSumKDR[CTS]-kdrs[pCT])/(teamCounts[TS]+1), (teamSumKDR[TS]+kdrs[pCT])/(teamCounts[CTS]-1) )
	}

	pT = searchBestTransfer(TS, 1)	
	if (pT==0)
		pT = searchBestTransfer(TS, 0)	
	if (pT==0)
		debug_print("No TS transfert would improve balance!")
	else
	{
		get_user_name(pT, nameT, 31)
		debug_print("Transfering %s to the Counter-Terros would improve balance: CT=%.2f T=%.2f",
			nameT, (teamSumKDR[CTS]+kdrs[pT])/(teamCounts[TS]-1), (teamSumKDR[TS]-kdrs[pT])/(teamCounts[CTS]+1) )
	}
	
	if ((winStreaks>1)||(roundCounter-lastRoundTB>2))
	{
		if (lastWinner==CTS) {
			if  (pCT!=0) {
				client_print(0, print_chat, "[HTB] Transfering %s to the Terros!", nameCT)				
				transferPlayer(pCT, TS)
			} else if (pT!=0) {
				client_print(0, print_chat, "[HTB] Transfering %s to the Counter-Terrorists!", nameT)				
				transferPlayer(pT, CTS)
			}	else {
				client_print(0, print_chat, "[HTB] No transfer will improve team balance!")				
			}			
		} else {
			if (pT!=0) {
				client_print(0, print_chat, "[HTB] Transfering %s to the Counter-Terrorists!", nameT)				
				transferPlayer(pT, CTS)
			} else if  (pCT!=0) {
				client_print(0, print_chat, "[HTB] Transfering %s to the Terros!", nameCT)				
				transferPlayer(pCT, TS)
			} else {
				client_print(0, print_chat, "[HTB] No transfer will improve team balance!")				
			}	
		}
	}
	
}

public searchBestTransfer(team, deadonly)
{
	new oteam = 3-team
	if (teamCounts[team]<2)
		// no sufficient players to make a transfert
		return 0

	if (teamCounts[oteam]==0) 
	{ // no player in opposite team: take the best kdr to the opposite team
		new Float:bestKDR=0.0, p=0
		for(new i=1; i<=MAX_PLAYERS; i++)
		{
			if ((playerTeam[i]==team)&&(kdrs[i]>bestKDR))
			{
				bestKDR = kdrs[i]
				p=i
			}
		}
		return p
	}

	// search for best transfert		
	new Float:bestBalance = fabs( teamSumKDR[CTS]/teamCounts[TS] - teamSumKDR[TS]/teamCounts[CTS] )
	// debug_print("   Initial balance: %.2f", bestBalance)
	new p = 0
	new divTeam = teamCounts[oteam]+1, divOTeam = teamCounts[team]-1
	for(new i=1; i<=MAX_PLAYERS; i++) {
		if ( (playerTeam[i]==team) && (roundCounter-lastRoundSwitched[i]>3) && ((deadonly+is_user_alive(i))!=2) )
		{
			new Float:b = fabs( (teamSumKDR[team]-kdrs[i])/divTeam - (teamSumKDR[oteam]+kdrs[i])/divOTeam ) 
			new name[32]
			get_user_name(i, name, 31)
			// debug_print("   Transfering %s to the %s : balance=%.2f", name, teamName[oteam], b)
			if (b < bestBalance ) {
				bestBalance = b
				p = i
			}
		}
	}
	return p		
}

