/*
  ,------------------------------------------------------------
  |    HP Plugin for AMX
  |-------------------------------------------------
  | Author:      NetRipper
  |              #Choke @ (irc.)Quakenet(.org)
  |              plugin_hp@netripper.tmfweb.nl
  |
  | Description: When activated this plugin shows the health, weapon used
  |              and ammo left in the clip of the killer that killed you
  |              (the killer). This is both showed in console for later
  |              reference and using the 'tsay' style.
  |              Difference between this plugin and StatsMe or another
  |              stats module plugin is that it doesn't cost much CPU
  |              as it doesn't have to calculate many seperate things.
  |
  | Commands:    amx_hp [1/0]
  |
  | Access:      Everyone      can check status
  |              ADMIN_LEVEL_A can enable/disable plugin
  |
  | Thanks to:   Sir Drink a lot, for the logd version of this plugin
  |              and thus giving me the inspiration to make it for AMX.
  |
  | Changes:     Monday 2nd June 2003, v0.2:
  |              - Fixed crashing
  |              - Fixed showing one too many bullets left in clip
  |              - Using AMX's default log files
  |              Sunday 1st June 2003, v0.1:
  |              - First release
  `-------------------
*/

#include <amxmod>
#include <amxmisc>

// HP Plugin enabled by default
new hpstatus = 1;
new logfilename[256];

// Return current setting or set new value
public handle_hpstatus(id,level,cid) {

    // No switches given
    if (read_argc() < 2) {
        new status[50];
        if (hpstatus == 1) {
            copy(status, 49, "Enabled");
        }
        else {
            copy(status, 49, "Disabled");
        }
        client_print(id,print_console,"[AMX] HP - status: %s", status);
        if (cmd_access(id,ADMIN_LEVEL_A,cid,0)) 
           client_print(id,print_console,"[AMX] HP - usage: amx_hp [0/1]");
        return PLUGIN_HANDLED;
    }

    // If you don't have enough rights, you can't change anything
    if (!cmd_access(id,ADMIN_LEVEL_A,cid,0))
        return PLUGIN_HANDLED;
    
    new arg[3];
    read_argv(1,arg,2);

    if (equal(arg,"0",1)) {
        hpstatus = 0;
        client_print(0,print_chat,"[AMX] HP - Plugin has been disabled, you will NOT see killers HP after you die.");
    }
    else if (equal(arg,"1",1)) {
        hpstatus = 1;
        client_print(0,print_chat,"[AMX] HP - Plugin has been enabled, you will see killers HP after you die.");
    }

    new authid[17],name[33];
    get_user_authid(id,authid,16);
    get_user_name(id,name,32);

    log_to_file(logfilename,"HP: ^"%s<%d><%s><>^" amx_hp %s",name,get_user_userid(id),authid,arg);
    return PLUGIN_HANDLED;
}

public event_hpdeath() {
    // If plugin is disabled, skip the code
    if (hpstatus <= 0)
       return PLUGIN_CONTINUE;

    // Gather information
    new killerweapon[33], killerweaponid, killerweaponammo, killerweaponclip;
    new killerhealth, killername[33], bulletsleft[50];
    new linet1[129],linet2[129],linec[129];

    new killer = read_data(1);
    new victim = read_data(2);

    // In case of self-kill, continue
    if (killer == victim) return PLUGIN_CONTINUE;
    
    get_user_name(killer,killername,32);
    killerhealth = get_user_health(killer);
    killerweaponid = get_user_weapon(killer,killerweaponclip,killerweaponammo);
    get_weaponname(killerweaponid,killerweapon,32);

    // Remove the "weapon_" part from weaponname
    replace(killerweapon,31,"weapon_","");

    // Fix for 'one too many bullet in clip when displaying'
    killerweaponclip--;

    // Determine if there are bullets left, if so check if bullet or bullets
    if (killerweaponclip >= 0) {
        if (killerweaponclip == 1) format(bulletsleft, 49, " (%d bullet left in clip)", killerweaponclip);
        else format(bulletsleft, 49, " (%d bullets left in clip)", killerweaponclip);
    }
    else {
        format(killerweapon, 32, "");
    }

    format(linet1, 127, "%s has %d HP left after killing you", killername, killerhealth);
    format(linet2, 127, "with the %s%s", killerweapon, bulletsleft);
    format(linec, 127, "%s has %d HP left after killing you with the %s%s", killername, killerhealth, killerweapon, bulletsleft);

    // Display information using "tsay" style
    set_hudmessage(200, 200, 0, 0.1, 0.75, 0, 6.0, 12.0, 1.0, 1.0, 3);
	show_hudmessage(victim,"%s",linet1);
    set_hudmessage(200, 200, 0, 0.1, 0.77, 0, 6.0, 12.0, 1.0, 1.0, 4);
	show_hudmessage(victim,"%s",linet2);

    // Show information in console for later reference
    client_print(victim,print_console,linec);

    return PLUGIN_CONTINUE;
}

public plugin_init() {
    register_plugin("plugin_hp", "0.1", "NetRipper");
    register_event("DeathMsg","event_hpdeath","a");
    register_concmd("amx_hp", "handle_hpstatus",-1,"[0/1]");

    get_time("addons/amx/logs/admin%m%d.log",logfilename,255) 

    return PLUGIN_CONTINUE;
}
