
   /*
   *    AMXMOD script.
   *    (plugin_csevents.sma)
   *    by mike_cao <mike@mikecao.com>
   *    This file is provided as is (no warranties).
   *
   *    This plugin does a centersay announcement
   *    for events in counter-strike. Event are configured
   *    with the cvar "amx_csevents" using these flags:
   *
   *    a - Bomb exploded
   *    b - Bomb planted
   *    c - Bomb defused
   *    d - Bomb being planted
   *    e - Bomb being defused
   *    f - Player spawned with bomb
   *    g - Player picked up bomb
   *    h - Player dropped bomb
   *    i - Audio bomb countdown
   *    j - Hostage touched
   *    k - Hostage rescued
   *    l - Hostage killed (with punishment)
   *    m - Headshots (special effects)
   *    n - Knife kills (multiple messages)
   *    o - Grenade kills (multiple messages)
   *    p - Grenade suicides
   *    q - Player spawned as VIP
   *    r - VIP killed
   *    s - VIP escaped
   *    t - VIP failed to escape
   *    u - First kill (bonus frag)
   *    v - Killstreak
   *
   */
   #include <amxmod>
   #include <logd>
   #define MAX_NAME_LENGTH 32
   #define MAX_VAR_LENGTH 64
   #define MAX_PLAYERS 32
   #define MAX_WEAPONS 32
   #define MAX_TEXT_LENGTH 512
   #define EVENT_BOMB_EXPLODE        (1<<0) // "a"
   #define EVENT_BOMB_PLANTED         (1<<1) // "b"
   #define EVENT_BOMB_DEFUSED         (1<<2) // "c"
   #define EVENT_BOMB_PLANTING     (1<<3) // "d"
   #define EVENT_BOMB_DEFUSING     (1<<4) // "e"
   #define EVENT_BOMB_SPAWN         (1<<5) // "f"
   #define EVENT_BOMB_PICKUP         (1<<6) // "g"
   #define EVENT_BOMB_DROPPED         (1<<7) // "h"
   #define EVENT_BOMB_COUNT         (1<<8) // "i"
   #define EVENT_HOSTAGE_TOUCHED    (1<<9) // "j"
   #define EVENT_HOSTAGE_RESCUED    (1<<10) // "k"
   #define EVENT_HOSTAGE_KILLED    (1<<11) // "l"
   #define EVENT_HEADSHOT            (1<<12) // "m"
   #define EVENT_KNIFE                (1<<13) // "n"
   #define EVENT_HEKILL            (1<<14) // "o"
   #define EVENT_HESUICIDE            (1<<15) // "p"
   #define EVENT_VIP_SPAWN            (1<<16) // "q"
   #define EVENT_VIP_KILLED        (1<<17) // "r"
   #define EVENT_VIP_ESCAPED        (1<<18) // "s"
   #define EVENT_VIP_FAILED        (1<<19) // "t"
   #define EVENT_FIRST_KILL        (1<<20) // "u"
   #define EVENT_KILLSTREAK        (1<<21) // "v"
   /************************************************************
   * CONFIG
   ************************************************************/
   // Number of hostages killed before punishment
   #define DEFAULT_HOSTAGE_KILLS        2
   // No name change allowed this many seconds after round starts
   #define DEFAULT_NAMECHANGE_TIME        30
   // Extra frags for getting the first kill
   #define DEFAULT_FIRST_KILL_BONUS    1
   // Text for knife kill messages
   // You have to adjust the index number if you change
   // the number of messages
   new gKnifeIndex = 3
   new gKnifeText[3][] = {
       "got owned by",
       "was sliced and diced by",
       "got shanked by"
   }
   // Text for grenade kill messages
   // You have to adjust the index number if you change
   // the number of messages
   new gGrenadeIndex = 4
   new gGrenadeText[4][] = {
       "got blown to bits by",
       "felt the heat of",
       "was burned by",
       "got toasted by"
   }
   /************************************************************
   * MAIN
   ************************************************************/
   new gPlanter, gVIP, gBombTimer, gFirstKill, gDefuser
   new Float:gSpawnTime
   new gHostageIndex[MAX_PLAYERS+1]
   new gKillIndex[MAX_PLAYERS+1]
   new gNameIndex[MAX_PLAYERS+1][MAX_NAME_LENGTH]
   new gChangeIndex[MAX_PLAYERS+1]
   /************************************************************
   * EVENT FUNCTIONS
   ************************************************************/
   // Player Events
   public logd_player_action(id,action[])
   {
       new sName[MAX_NAME_LENGTH]
       new sMsg[MAX_TEXT_LENGTH]
       new iFlags = get_event_flags()
       get_user_name(id,sName,MAX_NAME_LENGTH)
       // Bomb planted
       if(equal(action,"Planted_The_Bomb")) {
           if (iFlags&EVENT_BOMB_PLANTED) {
               format(sMsg,MAX_TEXT_LENGTH,"%s planted the bomb!",sName)
               display_msg(sMsg,255,0,0)
               if (iFlags&EVENT_BOMB_COUNT) {
                   gBombTimer = get_cvar_num("mp_c4timer")
                   set_task(1.0,"event_bomb_count",80,"",0,"b")
               }
           }
       }
       // Bomb defused
       else if (equal(action,"Defused_The_Bomb")) {
           if (iFlags&EVENT_BOMB_DEFUSED) {
               format(sMsg,MAX_TEXT_LENGTH,"%s  defused  the bomb with %i
   seconds left!",sName,gBombTimer)
               display_msg(sMsg,0,100,255)
           }
       }
       // Bomb defusing with a kit
       else if (equal(action,"Begin_Bomb_Defuse_With_Kit")) {
           if (iFlags&EVENT_BOMB_DEFUSING) {
               format(sMsg,MAX_TEXT_LENGTH,"%s  is defusing the bomb with
   a kit...",sName)
               display_msg(sMsg,0,100,255)
           }
           gDefuser = id
       }
       // Bomb defusing without a kit
       else if (equal(action,"Begin_Bomb_Defuse_Without_Kit")) {
           if (iFlags&EVENT_BOMB_DEFUSING) {
               format(sMsg,MAX_TEXT_LENGTH,"%s  is defusing the bomb with
   his bare hands...",sName)
               display_msg(sMsg,0,100,255)
           }
           gDefuser = id
       }
       // Spawned with the bomb
       else if (equal(action,"Spawned_With_The_Bomb")) {
           if (iFlags&EVENT_BOMB_SPAWN) {
               format(sMsg,MAX_TEXT_LENGTH,"%s has the bomb!",sName)
               display_msg(sMsg,0,200,0)
           }
           gPlanter = id
       }
       // Dropped bomb
       else if (equal(action,"Dropped_The_Bomb")) {
           if (iFlags&EVENT_BOMB_DROPPED) {
               format(sMsg,MAX_TEXT_LENGTH,"%s dropped the bomb!",sName)
               display_msg(sMsg,0,200,0)
           }
       }
       // Picked up bomb
       else if (equal(action,"Got_The_Bomb")) {
           if (iFlags&EVENT_BOMB_PICKUP) {
               format(sMsg,MAX_TEXT_LENGTH,"%s      picked     up     the
   bomb!",sName)
               display_msg(sMsg,0,200,0)
           }
           gPlanter = id
       }
       // Hostage touched
       else if (equal(action,"Touched_A_Hostage")) {
           if (iFlags&EVENT_HOSTAGE_TOUCHED) {
               format(sMsg,MAX_TEXT_LENGTH,"%s found a hostage!",sName)
               display_msg(sMsg,0,50,128)
           }
       }
       // Hostage rescued
       else if (equal(action,"Rescued_A_Hostage")) {
           if (iFlags&EVENT_HOSTAGE_RESCUED) {
               format(sMsg,MAX_TEXT_LENGTH,"%s rescued a hostage!",sName)
               display_msg(sMsg,0,100,255)
           }
       }
       // Hostage killed
       else if (equal(action,"Killed_A_Hostage")) {
           if (iFlags&EVENT_HOSTAGE_KILLED) {
               gHostageIndex[id] += 1
               if (gHostageIndex[id] >= DEFAULT_HOSTAGE_KILLS) {
                   user_kill(id,1)
                   format(sMsg,MAX_TEXT_LENGTH,"%s   was   punished   for
   killing hostages!",sName)
                   display_msg(sMsg,255,175,0)

               }
               else {
                   format(sMsg,MAX_TEXT_LENGTH,"%s        killed        a
   hostage!",sName)
                   display_msg(sMsg,255,175,0)
               }
           }
       }
       // VIP spawn
       else if (equal(action,"Became_VIP")) {
           if (iFlags&EVENT_VIP_SPAWN) {
               format(sMsg,MAX_TEXT_LENGTH,"%s     has     become     the
   VIP!",sName)
               display_msg(sMsg,0,200,0)
           }
           gVIP = id
       }
       // VIP assassinated
       else if (equal(action,"Assassinated_The_VIP")) {
           if (iFlags&EVENT_VIP_KILLED) {
               new sNameVIP[MAX_NAME_LENGTH]
               get_user_name(gVIP,sNameVIP,MAX_NAME_LENGTH)
               format(sMsg,MAX_TEXT_LENGTH,"%s    assassinated   %s   the
   VIP!",sName,sNameVIP)
               display_msg(sMsg,255,0,0)
           }
       }
       // VIP escaped
       else if (equal(action,"Escaped_As_VIP")) {
           if (iFlags&EVENT_VIP_ESCAPED) {
               format(sMsg,MAX_TEXT_LENGTH,"%s      the      VIP      has
   escaped!",sName)
               display_msg(sMsg,0,100,255)
           }
       }
       return PLUGIN_HANDLED
   }
   // Bomb planting
   public event_bomb_planting(id)
   {
       new sName[MAX_NAME_LENGTH]
       new sMsg[MAX_TEXT_LENGTH]
       new iFlags = get_event_flags()
       if (read_data(1) == 3) {
           if (iFlags&EVENT_BOMB_PLANTING) {
               get_user_name(gPlanter,sName,MAX_NAME_LENGTH)
               format(sMsg,MAX_TEXT_LENGTH,"%s     is     planting    the
   bomb...",sName)
               display_msg(sMsg,255,0,0)
           }
       }
       return PLUGIN_CONTINUE
   }
   // Bomb counter
   public event_bomb_count()
   {
       new iFlags = get_event_flags()
       gBombTimer -= 1
       if (gBombTimer > 0) {
           if (iFlags&EVENT_BOMB_COUNT) {
               new sSound[MAX_VAR_LENGTH]
               if (gBombTimer == 30 || gBombTimer == 20) {
                   num_to_word(gBombTimer,sSound,MAX_VAR_LENGTH)
                   client_cmd(0,"spk      ^"vox/%s      seconds     until
   explosion^"",sSound)
               }
               else if (gBombTimer < 11) {
                   num_to_word(gBombTimer,sSound,MAX_VAR_LENGTH)
                   client_cmd(0,"spk ^"vox/%s^"",sSound)
               }
           }
       }
       else {
           remove_task(80)
       }
       return PLUGIN_CONTINUE
   }
   // Death events
   public event_death()
   {
       new iFlags = get_event_flags()
       new iKiller = read_data(1)
       new iVictim = read_data(2)
       new iHeadshot = read_data(3)
       new sMsg[MAX_TEXT_LENGTH]
       new sWeapon[MAX_NAME_LENGTH]
       new sNameKiller[MAX_NAME_LENGTH], sNameVictim[MAX_NAME_LENGTH]
       read_data(4,sWeapon,MAX_NAME_LENGTH)
       get_user_name(iKiller,sNameKiller,MAX_NAME_LENGTH)
       get_user_name(iVictim,sNameVictim,MAX_NAME_LENGTH)

       if (iKiller != iVictim) {
           // First kill
           if      (iFlags&EVENT_FIRST_KILL     &&     !gFirstKill     &&
   (get_user_team(iKiller) != get_user_team(iVictim))) {
               format(sMsg,MAX_TEXT_LENGTH,"%s   got   the   first  kill!
   (+%i)",sNameKiller,DEFAULT_FIRST_KILL_BONUS)
               display_msg(sMsg,255,175,0)
               set_user_frags(iKiller,get_user_frags(iKiller)+DEFAULT_FIR
   ST_KILL_BONUS)
               gFirstKill = 1
           }
           // Headshot
           if (iFlags&EVENT_HEADSHOT && iHeadshot) {
               new iOrigin[3]
               get_user_origin(iVictim,iOrigin)
               display_fx_hs(iOrigin)
           }
           // Grenade kills
           if     (iFlags&EVENT_HEKILL    &&    equal(sWeapon,"grenade"))
   {
               format(sMsg,MAX_TEXT_LENGTH,"%s          %s          %s^'s
   grenade!",sNameVictim,gGrenadeText[random_num(0,gGrenadeIndex-1)],sNam
   eKiller)
               display_msg(sMsg,255,175,0)
           }
           // Knife kills
           if (iFlags&EVENT_KNIFE && equal(sWeapon,"knife")) {
               format(sMsg,MAX_TEXT_LENGTH,"%s          %s          %s^'s
   knife!",sNameVictim,gKnifeText[random_num(0,gKnifeIndex-1)],sNameKille
   r)
               display_msg(sMsg,255,175,0)
           }
           // Killstreak
           if (iFlags&EVENT_KILLSTREAK) {
               gKillIndex[iKiller] += 1
               gKillIndex[iVictim] = 0
               if (gKillIndex[iKiller] == 3) {
                   format(sMsg,MAX_TEXT_LENGTH,"%s  got  %i  kills  in  a
   row!",sNameKiller,gKillIndex[iKiller])
                   display_msg(sMsg,200,200,200)
               }
               else if (gKillIndex[iKiller] == 10) {
                   format(sMsg,MAX_TEXT_LENGTH,"%s  got  %i  kills  in  a
   row!",sNameKiller,gKillIndex[iKiller])
                   display_msg(sMsg,255,200,100)
               }
               else if (gKillIndex[iKiller] == 20) {
                   format(sMsg,MAX_TEXT_LENGTH,"%s  got  %i  kills  in  a
   row!",sNameKiller,gKillIndex[iKiller])
                   display_msg(sMsg,255,175,0)
               }
           }
       }
       else {
           // HE Suicide
           if (iFlags&EVENT_HESUICIDE && equal(sWeapon,"grenade")) {
               format(sMsg,MAX_TEXT_LENGTH,"%s   got  owned  by  his  own
   grenade!",sNameVictim)
               display_msg(sMsg,200,200,200)
           }
       }
       return PLUGIN_CONTINUE
   }
   // Team events
   public logd_team_action(team[],action[],optional[])
   {
       new iFlags = get_event_flags()
       new sName[MAX_NAME_LENGTH]
       new sMsg[MAX_TEXT_LENGTH]
       if (equal(team,"TERRORIST")) {
           if (equal(action,"Target_Bombed")) {
               if (iFlags&EVENT_BOMB_EXPLODE) {
                   get_user_name(gPlanter,sName,MAX_NAME_LENGTH)
                   format(sMsg,MAX_TEXT_LENGTH,"%s^'s                bomb
   exploded!",sName)
                   display_msg(sMsg,0,200,0)
                   remove_task(80)
                   if (gDefuser) {
                       get_user_name(gDefuser,sName,MAX_NAME_LENGTH)
                       format(sMsg,MAX_TEXT_LENGTH,"%s  failed  to defuse
   the bomb!",sName)
                       display_msg_sub(sMsg,0,200,0)
                   }
               }
           }
           else if (equal(action,"VIP_Assassinated")) {
               // Nothing
           }
       }
       else if (equal(team,"CT")) {
           if (equal(action,"VIP_Not_Escaped")) {
               if (iFlags&EVENT_VIP_FAILED) {
                   get_user_name(gVIP,sName,MAX_NAME_LENGTH)
                   format(sMsg,MAX_TEXT_LENGTH,"%s   the  VIP  failed  to
   escape!",sName)
                   display_msg(sMsg,0,200,0)
               }
           }
           else if (equal(action,"Bomb_Defused")) {
               // Nothing
           }
           else if (equal(action,"Hostages_Not_Rescued")) {
               // Nothing
           }
           else if (equal(action,"All_Hostages_Rescued")) {
               // Nothing
           }
       }
       return PLUGIN_HANDLED
   }
   // Round events
   public logd_world_action(action[])
   {
       if (equal(action,"Round_Start")) {
           gSpawnTime = get_gametime()
           gBombTimer = 0
       }
       else if (equal(action,"Round_End")) {
           gPlanter = 0
           gVIP = 0
           gBombTimer = 0
           gFirstKill = 0
           gDefuser = 0
       }
       return PLUGIN_HANDLED
   }
   // Name change
   public logd_name_change(id,newname[])
   {
       if (get_gametime() > (gSpawnTime + DEFAULT_NAMECHANGE_TIME)) {
           client_cmd(id,"name ^"%s^"",gNameIndex[id])
           if (gChangeIndex[id]==1) {
               client_print(id,print_console,"*  You  can't  change  your
   name %i seconds after the round has started.",DEFAULT_NAMECHANGE_TIME)
               gChangeIndex[id] = 0
           }
           else {
               gChangeIndex[id] = 1
           }
       }
       else {
           copy(gNameIndex[id],MAX_NAME_LENGTH,newname)
       }
       return PLUGIN_HANDLED
   }
   /************************************************************
   * HELPER FUNCTIONS
   ************************************************************/
   public display_msg(msg[],r,g,b)
   {
       set_hudmessage(r,g,b,-1.0,0.30,0,6.0,6.0,0.5,0.15,1)
       show_hudmessage(0,msg)
   }
   public display_msg_sub(msg[],r,g,b)
   {
       set_hudmessage(r,g,b,-1.0,0.35,0,6.0,6.0,0.5,0.15,2)
       show_hudmessage(0,msg)
   }
   public display_fx_hs(origin[3])
   {
       // Blood spray, 5 times
       for (new i = 0; i < 5; i++) {
           message_begin(MSG_BROADCAST,SVC_TEMPENTITY)
           write_byte(101)
           write_coord(origin[0])
           write_coord(origin[1])
           write_coord(origin[2]+30)
           write_coord(random_num(-20,20)) // x
           write_coord(random_num(-20,20)) // y
           write_coord(random_num(50,300)) // z
           write_byte(70) // color
           write_byte(random_num(100,200)) // speed
           message_end()
       }
   }
   public get_event_flags()
   {
       new sFlags[24]
       get_cvar_string("amx_csevents",sFlags,24)
       return read_flags(sFlags)
   }
   /************************************************************
   * PLUGIN FUNCTIONS
   ************************************************************/
   public client_connect(id)
   {
       new sName[MAX_NAME_LENGTH]
       get_user_name(id,sName,MAX_NAME_LENGTH)
       copy(gNameIndex[id],MAX_NAME_LENGTH,sName)
       gKillIndex[id] = 0
       gHostageIndex[id] = 0
       return PLUGIN_CONTINUE
   }
   public client_disconnect(id)
   {
       copy(gNameIndex[id],MAX_NAME_LENGTH,"")
       gKillIndex[id] = 0
       gHostageIndex[id] = 0
       return PLUGIN_CONTINUE
   }
   public plugin_init()
   {
       register_plugin("Plugin CS Events","1.1","mike_cao")
       // LogD events
       logd_init()

       // General events
       register_event("DeathMsg","event_death","a")
       register_event("BarTime","event_bomb_planting","be","1=3")

       // Cvars
       register_cvar("amx_csevents","abcdefghijklmnopqrstuv")
   }
