// AMX Mod script. 
//  
// Yet Another Bot Balancing
// (c) Copyright 2004, Serj Poznanski <_serj@ukr.net>
// 
// This file is provided as is (no warranties).
//
// Cvars:
//   amx_yabb_enable 1/0 - enable/disable bot balancing 
//   amx_yabb_ct 3 - balance number of CT players
//   amx_yabb_t 3  - balance number of T players
//   amx_yabb_addct "bot_add_ct" - adding CT bot command 
//   amx_yabb_addt "bot_add_t" - adding T bot command 
//
// Server Commands:
//   amx_yabb_stop - disable YABB and kick all bots 
//   amx_yabb_start - enable YABB and add bots
//   amx_yabb_inc_ct - increase CT team 
//   amx_yabb_dec_ct - decrease CT team 
//   amx_yabb_inc_t - increase T team 
//   amx_yabb_dec_t - decrease T team 
//

#include <amxmod>

new inifile[] = "addons/amx/yabb.ini"

public plugin_init() 
{
	register_plugin("YABB", "0.22", "S.Poznanski")

	register_cvar("amx_yabb_enable", "0")
	register_cvar("amx_yabb_ct", "3")
	register_cvar("amx_yabb_t", "3")
	register_cvar("amx_yabb_addct", "bot_add_ct")
	register_cvar("amx_yabb_addt", "bot_add_t")

	register_srvcmd("amx_yabb_stop", "yabb_stop")
	register_srvcmd("amx_yabb_start", "yabb_start")
	register_srvcmd("amx_yabb_inc_ct", "yabb_inc_ct")
	register_srvcmd("amx_yabb_dec_ct", "yabb_dec_ct")
	register_srvcmd("amx_yabb_inc_t", "yabb_inc_t")
	register_srvcmd("amx_yabb_dec_t", "yabb_dec_t")

	register_logevent("round_start", 2, "1=Round_Start")

	load_settings()

	return PLUGIN_CONTINUE
}

public yabb_stop()
{
	if (get_cvar_num("amx_yabb_enable") == 0)
		return PLUGIN_HANDLED

	set_cvar_num("amx_yabb_enable", 0)
	save_settings()

	set_cvar_num("amx_yabb_ct", 0)
	set_cvar_num("amx_yabb_t", 0)

	bot_balancing()

	return PLUGIN_HANDLED
}

public yabb_start()
{
	load_settings()
	set_cvar_num("amx_yabb_enable", 1)
	save_settings()

	bot_balancing()

	return PLUGIN_HANDLED
}

public yabb_inc_ct()
{
	new new_ct = get_cvar_num("amx_yabb_ct") + 1

	if ((new_ct + get_cvar_num("amx_yabb_t")) < get_maxplayers())
	{
		set_cvar_num("amx_yabb_ct", new_ct)
		bot_balancing()
		save_settings()
	}

	return PLUGIN_HANDLED
}

public yabb_dec_ct()
{
	new new_ct = get_cvar_num("amx_yabb_ct") - 1

	if (new_ct >= 0)
	{
		set_cvar_num("amx_yabb_ct", new_ct)
		bot_balancing()
		save_settings()
	}

	return PLUGIN_HANDLED
}

public yabb_inc_t()
{
	new new_t = get_cvar_num("amx_yabb_t") + 1

	if ((new_t + get_cvar_num("amx_yabb_ct")) < get_maxplayers())
	{
		set_cvar_num("amx_yabb_t", new_t)
		bot_balancing()
		save_settings()
	}

	return PLUGIN_HANDLED
}

public yabb_dec_t()
{
	new new_t = get_cvar_num("amx_yabb_t") - 1

	if (new_t >= 0)
	{
		set_cvar_num("amx_yabb_t", new_t)
		bot_balancing()
		save_settings()
	}

	return PLUGIN_HANDLED
}

public round_start()
{
	if (get_cvar_num("amx_yabb_enable") == 1) 
		bot_balancing()

	return PLUGIN_CONTINUE	
}

public bot_balancing()
{
	new ctplayers[32], tplayers[32], ctplayers_num, tplayers_num 
	new ctbots[32], tbots[32], ctbots_num, tbots_num

	new ct_num, t_num

	new ct_balance, t_balance
	new ct_addcommand[32], t_addcommand[32]
	
	ct_balance = get_cvar_num("amx_yabb_ct")
	t_balance = get_cvar_num("amx_yabb_t")
	get_cvar_string("amx_yabb_addct", ct_addcommand, 31)
	get_cvar_string("amx_yabb_addt", t_addcommand, 31)

	get_players(ctplayers, ctplayers_num, "ce", "CT")
	get_players(tplayers, tplayers_num, "ce", "TERRORIST")
	get_players(ctbots, ctbots_num, "de", "CT")
	get_players(tbots, tbots_num, "de", "TERRORIST")

	ct_num = ctplayers_num + ctbots_num
	t_num = tplayers_num + tbots_num
	
	new i, diff, kicknum, botsid

	if (ct_num < ct_balance)
	{
		diff = ct_balance - ct_num	
		for (i = 0; i < diff; ++i)
		{
			server_cmd(ct_addcommand)
		}
	}

	if (t_num < t_balance)
	{
		diff = t_balance - t_num	
		for (i = 0; i < diff; ++i)
		{
			server_cmd(t_addcommand)
		}
	}

	kicknum = 0
	if (ct_num > ct_balance)
	{
		diff = ct_num - ct_balance
		if (ctbots_num > 0)
		{
			if (ctbots_num  > diff)
			{
				kicknum = diff
			}
			else
			{
				kicknum = ctbots_num
			}

			for (i = 0; i < kicknum; ++i)
			{
				botsid = get_user_userid(ctbots[i])
				server_cmd("kick #%d", botsid)
			}
		}
	}

	kicknum = 0
	if (t_num > t_balance)
	{
		diff = t_num - t_balance
		if (tbots_num > 0)
		{
			if (tbots_num  > diff)
			{
				kicknum = diff
			}
			else
			{
				kicknum = tbots_num
			}

			for (i = 0; i < kicknum; ++i)
			{
				botsid = get_user_userid(tbots[i])
				server_cmd("kick #%d", botsid)
			}
		}
	}
}

save_settings()
{
	if (file_exists(inifile))
		delete_file(inifile)
	
	new text[256]

	if (!write_file(inifile,";Generated by YABB Plugin. Do not modify!^n;Value  Description"))
		return 0

	format(text, 255, "%d ;%s", get_cvar_num("amx_yabb_enable"), "YABB enable/disable")
	write_file(inifile,text)

	format(text, 255, "%d ;%s", get_cvar_num("amx_yabb_ct"), "CT team")
	write_file(inifile,text)

	format(text, 255, "%d ;%s", get_cvar_num("amx_yabb_t"), "T team")
	write_file(inifile,text)

	return 1
}

load_settings()
{
	if (!file_exists(inifile)) 
		return 0

	new text[256], value[4], len, count = 0, pos = 0

	while (read_file(inifile,pos++,text,255,len))
	{
		if (text[0] == ';') 
			continue // line is a comment
		
		parse(text, value, 3)

		if (count == 0)
		{
			set_cvar_num("amx_yabb_enable", strtonum(value))
			++count
			continue
		}
		
		if (count == 1)
		{
			set_cvar_num("amx_yabb_ct", strtonum(value))
			++count
			continue
		}
		
		if (count == 2)
		{
			set_cvar_num("amx_yabb_t", strtonum(value))
			++count
			continue
		}
		
	}
	return 1
}
